@@ -1,4 +1,4 @@
-/* $Id: acconfig.h,v 1.79 1997/11/09 13:55:55 joda Exp $ */
+/* $Id: acconfig.h,v 1.81 1997/12/09 13:27:49 joda Exp $ */
 
 /* Define this to the type ssize_t should be */
 #undef ssize_t
@@ -112,6 +112,16 @@
 /* define if you need a declaration for __progname */
 #undef HAVE___PROGNAME_DECLARATION
 
+/* Define to isoc_realloc if you have a broken realloc */
+#undef BROKEN_REALLOC
+#ifdef BROKEN_REALLOC
+#define realloc(X, Y) isoc_realloc((X), (Y))
+#define isoc_realloc(X, Y) ((X) ? realloc((X), (Y)) : malloc(Y))
+#endif
+
+/* define if you don't want to use mmap (even if it exists) */
+#undef NO_MMAP
+
 @BOTTOM@
 
 #undef HAVE_INT8_T
