@@ -9,19 +9,19 @@
 
 #include "adm_locl.h"
 
-RCSID("$Id: ext_srvtab.c,v 1.11 1996/11/17 20:22:57 assar Exp $");
+RCSID("$Id: ext_srvtab.c,v 1.14 1997/11/21 02:48:57 assar Exp $");
 
 static des_cblock master_key;
 static des_cblock session_key;
 static des_key_schedule master_key_schedule;
-char progname[] = "ext_srvtab";
 static char realm[REALM_SZ];
 
 static void
 usage(void)
 {
     fprintf(stderr, 
-	    "Usage: %s [-n] [-r realm] instance [instance ...]\n", progname);
+	    "Usage: %s [-n] [-r realm] instance [instance ...]\n",
+	    __progname);
     exit(1);
 }
 
@@ -33,20 +33,11 @@ StampOutSecrets(void)
     memset(master_key_schedule, 0, sizeof master_key_schedule);
 }
 
-static void
-Die(void)
-{
-    StampOutSecrets();
-    exit(1);
-}
-
 static void
 FWrite(void *p, int size, int n, FILE *f)
 {
-    if (fwrite(p, size, n, f) != n) {
-	printf("Error writing output file.  Terminating.\n");
-	Die();
-    }
+    if (fwrite(p, size, n, f) != n)
+	errx(1, "Error writing output file.  Terminating.\n");
 }
 
 int
@@ -61,8 +52,11 @@ main(int argc, char **argv)
     int prompt = KDB_GET_PROMPT;
     int n, i;
     
+    set_progname (argv[0]);
     memset(realm, 0, sizeof(realm));
     
+    atexit(StampOutSecrets);
+
     /* Parse commandline arguments */
     if (argc < 2)
 	usage();
@@ -87,18 +81,14 @@ main(int argc, char **argv)
 		usage();
 	    else
 		if (!k_isinst(argv[i])) {
-		fprintf(stderr, "%s: bad instance name: %s\n",
-			progname, argv[i]);
-		usage();
+		  warnx("bad instance name: %s", argv[i]);
+		  usage();
 	    }
 	}
     }
 
-    if (kdb_get_master_key (prompt, &master_key, master_key_schedule) != 0) {
-      fprintf (stderr, "Couldn't read master key.\n");
-      fflush (stderr);
-      exit(1);
-    }
+    if (kdb_get_master_key (prompt, &master_key, master_key_schedule) != 0)
+      errx (1, "Couldn't read master key.");
 
     if (kdb_verify_master_key (&master_key, master_key_schedule, stderr) < 0) {
       exit(1);
@@ -107,18 +97,16 @@ main(int argc, char **argv)
     /* For each arg, search for instances of arg, and produce */
     /* srvtab file */
     if (!realm[0])
-	if (krb_get_lrealm(realm, 1) != KSUCCESS) {
-	    fprintf(stderr, "%s: couldn't get local realm\n", progname);
-	    exit(1);
-	}
+	if (krb_get_lrealm(realm, 1) != KSUCCESS)
+	  errx (1, "couldn't get local realm");
     umask(077);
 
     for (arg = 1; arg < argc; arg++) {
 	if (argv[arg][0] == '-')
 	    continue;
-	sprintf(fname, "%s-new-srvtab", argv[arg]);
+	snprintf(fname, sizeof(fname), "%s-new-srvtab", argv[arg]);
 	if ((fout = fopen(fname, "w")) == NULL) {
-	    fprintf(stderr, "Couldn't create file '%s'.\n", fname);
+	    warn("Couldn't create file '%s'.", fname);
 	    fopen_errs++;
 	    continue;
 	}
@@ -140,9 +128,5 @@ main(int argc, char **argv)
 	}
 	fclose(fout);
     }
-
-    StampOutSecrets();
-
     return fopen_errs;		/* 0 errors if successful */
-
 }
