@@ -40,7 +40,7 @@
 #include <config.h>
 #endif
 
-RCSID("$Id: pagsh.c,v 1.17 1997/05/02 14:27:38 assar Exp $");
+RCSID("$Id: pagsh.c,v 1.18 1997/12/05 04:25:15 assar Exp $");
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -85,6 +85,9 @@ main(int argc, char **argv)
   i = 0;
 
   args = (char **) malloc((argc + 10)*sizeof(char *));
+  if (args == NULL)
+      errx (1, "Out of memory allocating %u bytes",
+	    (argc + 10)*sizeof(char *));
   
   argv++;
 
@@ -98,12 +101,17 @@ main(int argc, char **argv)
     if(strcmp(*argv, "-c") ==  0) argv++;
     path = strdup(*argv++);
   }
+  if (path == NULL)
+      errx (1, "Out of memory copying path");
   
   p=strrchr(path, '/');
   if(p)
-    args[i++] = strdup(p+1);
+    args[i] = strdup(p+1);
   else
-    args[i++] = strdup(path);
+    args[i] = strdup(path);
+
+  if (args[i++] == NULL)
+      errx (1, "Out of memory copying arguments");
   
   while(*argv)
     args[i++] = *argv++;
