@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 1995, 1996 Kungliga Tekniska Hgskolan (Royal Institute
- * of Technology, Stockholm, Sweden).
+ * Copyright (c) 1995, 1996, 1997 Kungliga Tekniska Hgskolan
+ * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
  * Redistribution and use in source and binary forms, with or without
@@ -36,7 +36,7 @@
  * SUCH DAMAGE.
  */
 
-/* $Id: bsd_locl.h,v 1.82 1996/11/19 21:53:38 assar Exp $ */
+/* $Id: bsd_locl.h,v 1.101 1997/12/04 14:19:30 assar Exp $ */
 
 #define LOGALL
 #define KERBEROS
@@ -47,9 +47,6 @@
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
-#ifdef SOCKS
-#include <socks.h>
-#endif
 
 /* Any better way to test NO_MOTD? */
 #if (SunOS == 5) || defined(__hpux)
@@ -75,13 +72,26 @@
 #ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
 #endif
+#ifdef TIME_WITH_SYS_TIME
+#include <sys/time.h>
 #include <time.h>
+#elif defined(HAVE_SYS_TIME_H)
+#include <sys/time.h>
+#else
+#include <time.h>
+#endif
 #ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
 #endif
+
 #ifndef S_ISTXT
+#ifdef S_ISVTX
 #define S_ISTXT S_ISVTX
+#else
+#define S_ISTXT 0
 #endif
+#endif
+
 #ifdef HAVE_FCNTL_H
 #include <fcntl.h>
 #endif
@@ -89,9 +99,6 @@
 #include <dirent.h>
 #endif
 #include <signal.h>
-#ifdef HAVE_SYS_TIME_H
-#include <sys/time.h>
-#endif
 #ifdef HAVE_SYS_RESOURCE_H
 #include <sys/resource.h>
 #endif /* HAVE_SYS_RESOURCE_H */
@@ -101,12 +108,6 @@
 #ifdef HAVE_SYS_PARAM_H
 #include <sys/param.h>
 #endif
-#if defined(HAVE_SYS_IOCTL_H) && SunOS != 4
-#include <sys/ioctl.h>
-#endif
-#ifdef HAVE_SYS_IOCCOM_H
-#include <sys/ioccom.h>
-#endif
 
 #ifndef NCARGS
 #define NCARGS  0x100000 /* (absolute) max # characters in exec arglist */
@@ -144,6 +145,13 @@
 #include <netdb.h>
 #endif
 
+#if defined(HAVE_SYS_IOCTL_H) && SunOS != 4
+#include <sys/ioctl.h>
+#endif
+#ifdef HAVE_SYS_IOCCOM_H
+#include <sys/ioccom.h>
+#endif
+
 #ifdef HAVE_SYS_SOCKIO_H
 #include <sys/sockio.h>
 #endif
@@ -220,10 +228,10 @@
 
 #ifdef HAVE_UTMP_H
 #include <utmp.h>
-#endif
 #ifndef UT_NAMESIZE
 #define UT_NAMESIZE     sizeof(((struct utmp *)0)->ut_name)
 #endif
+#endif
 
 #ifdef HAVE_UTMPX_H
 #include <utmpx.h>
@@ -245,15 +253,27 @@
 
 #include <roken.h>
 
+#ifdef SOCKS
+#include <socks.h>
+#endif
+
 #include <des.h>
 #include <krb.h>
 #include <kafs.h>
 
-int kcmd(int *sock, char **ahost, u_int16_t rport, char *locuser, char *remuser, char *cmd, int *fd2p, KTEXT ticket, char *service, char *realm, CREDENTIALS *cred, Key_schedule schedule, MSG_DAT *msg_data, struct sockaddr_in *laddr, struct sockaddr_in *faddr, int32_t authopts);
+int kcmd(int *sock, char **ahost, u_int16_t rport, char *locuser,
+	 char *remuser, char *cmd, int *fd2p, KTEXT ticket,
+	 char *service, char *realm, CREDENTIALS *cred,
+	 Key_schedule schedule, MSG_DAT *msg_data,
+	 struct sockaddr_in *laddr, struct sockaddr_in *faddr,
+	 int32_t authopts);
 
-int krcmd(char **ahost, u_int16_t rport, char *remuser, char *cmd, int *fd2p, char *realm);
+int krcmd(char **ahost, u_int16_t rport, char *remuser, char *cmd,
+	  int *fd2p, char *realm);
 
-int krcmd_mutual(char **ahost, u_int16_t rport, char *remuser, char *cmd, int *fd2p, char *realm, CREDENTIALS *cred, Key_schedule sched);
+int krcmd_mutual(char **ahost, u_int16_t rport, char *remuser,
+		 char *cmd,int *fd2p, char *realm,
+		 CREDENTIALS *cred, Key_schedule sched);
 
 int klogin(struct passwd *pw, char *instance, char *localhost, char *password);
 
@@ -266,13 +286,8 @@ char *colon(char *cp);
 int okname(char *cp0);
 int susystem(char *s, int userid);
 
-#ifdef __GNUC__
-struct termios;
-struct winsize;
-int forkpty __P((int *amaster, char *name, struct termios *termp, struct winsize *winp));
-#else
-int forkpty ();
-#endif
+int forkpty(int *amaster, char *name,
+	    struct termios *termp, struct winsize *winp);
 
 #ifndef MODEMASK
 #define	MODEMASK	(S_ISUID|S_ISGID|S_ISTXT|S_IRWXU|S_IRWXG|S_IRWXO)
@@ -286,7 +301,7 @@ int forkpty ();
 #endif
 #include "pathnames.h"
 
-void stty_default __P((void));
+void stty_default (void);
 
 int utmpx_login(char *line, char *user, char *host);
 
@@ -296,10 +311,6 @@ void sysv_newenv(int argc, char **argv, struct passwd *pwd,
 		 char *term, int pflag);
 
 int login_access(char *user, char *from);
-#ifndef HAVE_IRUSEROK
-int iruserok(u_int32_t raddr, int superuser, const char *ruser,
-	     const char *luser);
-#endif
 void fatal(int f, const char *msg, int syserr);
 
 extern int LEFT_JUSTIFIED;
@@ -310,7 +321,7 @@ int des_enc_write(int fd,char *buf,int len,des_key_schedule sched,
 
 void sysv_defaults(void);
 void utmp_login(char *tty, char *username, char *hostname);
-void	 sleepexit __P((int));
+void sleepexit (int);
 
 #ifndef HAVE_SETPRIORITY
 #define setpriority(which, who, niceval) 0
@@ -320,16 +331,18 @@ void	 sleepexit __P((int));
 #define getpriority(which, who) 0
 #endif
 
+#ifdef HAVE_TERMIOS_H
 #include <termios.h>
+#endif
 
-#if SunOS == 4
 #ifndef _POSIX_VDISABLE
 #define _POSIX_VDISABLE 0
 #endif /* _POSIX_VDISABLE */
+#if SunOS == 4
 #include <sys/ttold.h>
 #endif
 
-#if defined(_AIX)
+#if defined(HAVE_SYS_TERMIO_H) && !defined(HAVE_TERMIOS_H)
 #include <sys/termio.h>
 #endif
 
@@ -351,3 +364,13 @@ int get_login_port(int kerberos, int encryption);
 int speed_t2int (speed_t);
 speed_t int2speed_t (int);
 void ip_options_and_die (int sock, struct sockaddr_in *);
+void warning(const char *fmt, ...)
+#ifdef __GNUC__
+__attribute__ ((format (printf, 1, 2)))
+#endif
+;
+
+char *clean_ttyname (char *tty);
+char *make_id (char *tty);
+void prepare_utmp (struct utmp *utmp, char *tty, char *username,
+		   char *hostname);
