@@ -39,7 +39,7 @@
 
 #include "bsd_locl.h"
 
-RCSID("$Id: login.c,v 1.106 1997/11/09 09:04:25 assar Exp $");
+RCSID("$Id: login.c,v 1.109 1997/12/09 10:36:26 joda Exp $");
 
 #ifdef OTP
 #include <otp.h>
@@ -212,7 +212,7 @@ main(int argc, char **argv)
 
 	*full_hostname = '\0';
 	domain = NULL;
-	if (k_gethostname(localhost, sizeof(localhost)) < 0)
+	if (gethostname(localhost, sizeof(localhost)) < 0)
 		syslog(LOG_ERR, "couldn't get local hostname: %m");
 	else
 		domain = strchr(localhost, '.');
@@ -525,10 +525,8 @@ main(int argc, char **argv)
 	    long t;
 	    const long maxcpu = 46116860184; /* some random constant */
 	    udb = getudbnam(pwd->pw_name);
-	    if(udb == UDB_NULL){
-		    warnx("Failed to get UDB entry.");
-		    exit(1);
-	    }
+	    if(udb == UDB_NULL)
+		errx(1, "Failed to get UDB entry.");
 	    t = udb->ue_pcpulim[UDBRC_INTER];
 	    if(t == 0 || t > maxcpu)
 		t = CPUUNLIM;
@@ -761,8 +759,8 @@ main(int argc, char **argv)
 	    char cell[64];
 	    k_setpag();
 	    if(k_afs_cell_of_file(pwd->pw_dir, cell, sizeof(cell)) == 0)
-		k_afsklog(cell, 0);
-	    k_afsklog(0, 0);
+		krb_afslog(cell, 0);
+	    krb_afslog(0, 0);
 	}
 
 	execlp(pwd->pw_shell, tbuf, 0);
