@@ -31,7 +31,7 @@
  * SUCH DAMAGE.
  *
  *	from: @(#)pathnames.h	5.2 (Berkeley) 4/9/90
- *	$Id: pathnames.h,v 1.21 1996/04/24 07:41:28 assar Exp $
+ *	$Id: pathnames.h,v 1.24 1997/11/03 11:17:19 assar Exp $
  */
 
 /******* First fix default path, we stick to _PATH_DEFPATH everywhere */
@@ -65,6 +65,9 @@
 #undef _PATH_RSH		/* Redifine rsh */
 #define _PATH_RSH	BINDIR  "/rsh"
 
+#undef _PATH_RCP		/* Redifine rcp */
+#define _PATH_RCP	BINDIR  "/rcp"
+
 #undef _PATH_LOGIN
 #define _PATH_LOGIN	BINDIR "/login"
 
@@ -102,6 +105,10 @@
 #define _PATH_HUSHLOGIN ".hushlogin"
 #endif /* _PATH_HUSHLOGIN */
 
+#ifndef _PATH_NOMAILCHECK
+#define _PATH_NOMAILCHECK ".nomailcheck"
+#endif /* _PATH_NOMAILCHECK */
+
 #ifndef _PATH_MOTDFILE
 #define _PATH_MOTDFILE "/etc/motd"
 #endif /* _PATH_MOTDFILE */
@@ -167,6 +174,10 @@
 #define _PATH_ETC_DEFAULT_LOGIN	"/etc/default/login"
 #endif
 
+#ifndef _PATH_ETC_ENVIRONMENT
+#define _PATH_ETC_ENVIRONMENT "/etc/environment"
+#endif
+
 /*
  * NeXT KLUDGE ALERT!!!!!!!!!!!!!!!!!!
  * Some sort of bug in the NEXTSTEP cpp.
@@ -178,6 +189,8 @@
 #define	_PATH_RLOGIN	"/usr/athena/bin/rlogin"
 #undef  _PATH_RSH
 #define _PATH_RSH	"/usr/athena/bin/rsh"
+#undef  _PATH_RCP
+#define _PATH_RCP	"/usr/athena/bin/rcp"
 #undef  _PATH_LOGIN
 #define _PATH_LOGIN	"/usr/athena/bin/login"
 #endif
