@@ -2,7 +2,7 @@
 
 #include "bsd_locl.h"
 
-RCSID("$Id: sysv_environ.c,v 1.18 1996/11/08 18:49:43 assar Exp $");
+RCSID("$Id: sysv_environ.c,v 1.22 1997/12/05 01:33:25 assar Exp $");
 
 #ifdef HAVE_ULIMIT_H
 #include <ulimit.h>
@@ -130,7 +130,7 @@ void sysv_newenv(int argc, char **argv, struct passwd *pwd,
 
     while (argc && *argv) {
 	if (strchr(*argv, '=') == 0) {
-	    sprintf(buf, "L%d", count++);
+	    snprintf(buf, sizeof(buf), "L%d", count++);
 	    setenv(buf, *argv, 1);
 	} else {
 	    for (cp = censored; cp->prefix; cp++)
@@ -149,10 +149,12 @@ void sysv_newenv(int argc, char **argv, struct passwd *pwd,
     /* Undocumented: HOME, MAIL and LOGNAME are always reset (SunOS 5.1). */
 
     setenv("HOME", pwd->pw_dir, 1);
-    strcpy (buf, KRB4_MAILDIR);
-    if (buf[strlen(buf) - 1] == '/')
-      buf[strlen(buf) - 1] = '\0';
-    sprintf(buf, "%s/%s", buf, pwd->pw_name);
+    {
+	char *sep = "/";
+	if(KRB4_MAILDIR[strlen(KRB4_MAILDIR) - 1] == '/')
+	    sep = "";
+	roken_concat(buf, sizeof(buf), KRB4_MAILDIR, sep, pwd->pw_name, NULL);
+    }
     setenv("MAIL", buf, 1);
     setenv("LOGNAME", pwd->pw_name, 1);
     setenv("USER", pwd->pw_name, 1);
@@ -182,7 +184,7 @@ void sysv_newenv(int argc, char **argv, struct passwd *pwd,
     if (default_ulimit) {
 	if (sscanf(default_ulimit, "%ld", &limit_val) == 1 && limit_val)
 	    if (ulimit(UL_SETFSIZE, limit_val) < 0)
-		printf("Could not set ULIMIT to %ld\n", limit_val);
+	        warn ("ulimit(UL_SETFSIZE, %ld)", limit_val);
     }
 #endif
     read_etc_environment();
