@@ -33,7 +33,7 @@
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
-RCSID("$Id: ftpd.c,v 1.86 1997/05/25 13:49:00 assar Exp $");
+RCSID("$Id: ftpd.c,v 1.95 1997/12/09 13:24:28 joda Exp $");
 #endif
 
 /*
@@ -135,7 +135,9 @@ RCSID("$Id: ftpd.c,v 1.86 1997/05/25 13:49:00 assar Exp $");
 #include <kafs.h>
 #include "roken.h"
 
+#ifdef OTP
 #include <otp.h>
+#endif
 
 #ifdef SOCKS
 #include <socks.h>
@@ -164,7 +166,7 @@ jmp_buf	errcatch, urgcatch;
 int	oobflag;
 int	logged_in;
 struct	passwd *pw;
-int	debug;
+int	debug = 0;
 int	ftpd_timeout = 900;    /* timeout after 15 minutes of inactivity */
 int	maxtimeout = 7200;/* don't allow idle time to be set beyond 2 hours */
 int	logging;
@@ -267,15 +269,17 @@ parse_auth_level(char *str)
 {
     char *p;
     int ret = 0;
-    char *foo;
+    char *foo = NULL;
 
     for(p = strtok_r(str, ",", &foo);
 	p;
 	p = strtok_r(NULL, ",", &foo)) {
 	if(strcmp(p, "user") == 0)
 	    ;
+#ifdef OTP
 	else if(strcmp(p, "otp") == 0)
 	    ret |= AUTH_PLAIN|AUTH_OTP;
+#endif
 	else if(strcmp(p, "ftp") == 0 ||
 		strcmp(p, "safe") == 0)
 	    ret |= AUTH_FTP;
@@ -426,7 +430,6 @@ main(int argc, char **argv)
 		syslog(LOG_WARNING, "setsockopt (IP_TOS): %m");
 #endif
 	data_source.sin_port = htons(ntohs(ctrl_addr.sin_port) - 1);
-	debug = 0;
 
 	/* set this here so it can be put in wtmp */
 	snprintf(ttyline, sizeof(ttyline), "ftp%u", (unsigned)getpid());
@@ -486,7 +489,7 @@ main(int argc, char **argv)
 		fclose(fd);
 		/* reply(220,) must follow */
 	}
-	k_gethostname(hostname, sizeof(hostname));
+	gethostname(hostname, sizeof(hostname));
 	reply(220, "%s FTP server (%s+%s) ready.", hostname, 
 	      version, krb4_version);
 	setjmp(errcatch);
@@ -552,7 +555,9 @@ sgetpwnam(char *name)
 static int login_attempts;	/* number of failed login attempts */
 static int askpasswd;		/* had user command, ask for passwd */
 static char curname[10];	/* current USER name */
+#ifdef OTP
 OtpContext otp_ctx;
+#endif
 
 /*
  * USER command.
@@ -636,20 +641,28 @@ user(char *name)
 	else {
 		char ss[256];
 
+#ifdef OTP
 		if (otp_challenge(&otp_ctx, name, ss, sizeof(ss)) == 0) {
 			reply(331, "Password %s for %s required.",
 			      ss, name);
 			askpasswd = 1;
-		} else if ((auth_level & AUTH_OTP) == 0) {
+		} else
+#endif
+		if ((auth_level & AUTH_OTP) == 0) {
 		    reply(331, "Password required for %s.", name);
 		    askpasswd = 1;
 		} else {
 		    char *s;
 		    
+#ifdef OTP
 		    if (s = otp_error (&otp_ctx))
 			lreply(530, "OTP: %s", s);
+#endif
 		    reply(530,
-			  "Only authorized, anonymous and OTP "
+			  "Only authorized, anonymous"
+#ifdef OTP
+			  " and OTP "
+#endif
 			  "login allowed.");
 		}
 
@@ -721,6 +734,7 @@ checkaccess(char *name)
     FILE *fd;
     int allowed = ALLOWED;
     char *user, *perm, line[BUFSIZ];
+    char *foo;
     
     fd = fopen(_PATH_FTPUSERS, "r");
     
@@ -728,10 +742,11 @@ checkaccess(char *name)
 	return allowed;
 
     while (fgets(line, sizeof(line), fd) != NULL)  {
-	user = strtok(line, " \t\n");
-	if (user[0] == '#')
+	foo = NULL;
+	user = strtok_r(line, " \t\n", &foo);
+	if (user == NULL || user[0] == '#')
 	    continue;
-	perm = strtok(NULL, " \t\n");
+	perm = strtok_r(NULL, " \t\n", &foo);
 	if (match(user, name) == 0){
 	    if(perm && strcmp(perm, "allow") == 0)
 		allowed = ALLOWED;
@@ -867,23 +882,29 @@ pass(char *passwd)
 	if (!guest) {		/* "ftp" is only account allowed no password */
 		if (pw == NULL)
 			rval = 1;	/* failure below */
+#ifdef OTP
 		else if (otp_verify_user (&otp_ctx, passwd) == 0) {
 		    rval = 0;
-		} else if((auth_level & AUTH_OTP) == 0) {
+		}
+#endif
+		else if((auth_level & AUTH_OTP) == 0) {
 		    char realm[REALM_SZ];
 		    if((rval = krb_get_lrealm(realm, 1)) == KSUCCESS)
 			rval = krb_verify_user(pw->pw_name, "", realm, 
 					       passwd, 1, NULL);
-		    if (rval == KSUCCESS ){
+		    if (rval == KSUCCESS ) {
+			chown (tkt_string(), pw->pw_uid, pw->pw_gid);
 			if(k_hasafs())
-			    k_afsklog(0, 0);
-		    }else 
+			    krb_afslog(0, 0);
+		    } else 
 			rval = unix_verify_user(pw->pw_name, passwd);
 		} else {
 		    char *s;
 		    
+#ifdef OTP
 		    if (s = otp_error(&otp_ctx))
 			lreply(530, "OTP: %s", s);
+#endif
 		}
 		memset (passwd, 0, strlen(passwd));
 
@@ -1282,7 +1303,7 @@ send_data(FILE *instr, FILE *outstr)
 		
 	case TYPE_I:
 	case TYPE_L:
-#ifdef HAVE_MMAP
+#if defined(HAVE_MMAP) && !defined(NO_MMAP)
 #ifndef MAP_FAILED
 #define MAP_FAILED (-1)
 #endif
@@ -1757,19 +1778,17 @@ dolog(struct sockaddr_in *sin)
 void
 dologout(int status)
 {
-
-	if (logged_in) {
-		seteuid((uid_t)0);
-		logwtmp(ttyline, "", "");
-		dest_tkt();
-		if(k_hasafs())
-		    k_unlog();
-	}
-	/* beware of flushing buffers after a SIGPIPE */
+    transflag = 0;
+    if (logged_in) {
+	seteuid((uid_t)0);
+	logwtmp(ttyline, "", "");
+	cond_kdestroy();
+    }
+    /* beware of flushing buffers after a SIGPIPE */
 #ifdef XXX
-	exit(status);
+    exit(status);
 #else
-	_exit(status);
+    _exit(status);
 #endif	
 }
 
