@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 1995, 1996 Kungliga Tekniska Hgskolan (Royal Institute
- * of Technology, Stockholm, Sweden).
+ * Copyright (c) 1995, 1996, 1997 Kungliga Tekniska Hgskolan
+ * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
  * Redistribution and use in source and binary forms, with or without
@@ -40,7 +40,7 @@
 #include <config.h>
 #endif
 
-RCSID("$Id: kauth.c,v 1.12 1996/11/17 21:08:25 assar Exp $");
+RCSID("$Id: kauth.c,v 1.17 1997/12/09 10:36:32 joda Exp $");
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -71,6 +71,8 @@ static time_t local_time;
 
 static krb_principal pr;
 
+static int do_destroy_tickets = 1;
+
 static int
 save_tkt(char *user, char *instance, char *realm, void *arg, 
 	 int (*key_proc)(char*, char*, char*, void*, des_cblock*), KTEXT *cipp)
@@ -184,7 +186,8 @@ store_ticket(KTEXT cip)
     return(kerror);
 }
 
-void kauth(char *principal, char *ticket)
+void
+kauth(char *principal, char *ticket)
 {
     char *p;
     int ret;
@@ -209,8 +212,10 @@ void kauth(char *principal, char *ticket)
 	    memset(&cip, 0, sizeof(cip));
 	    return;
 	}
+	do_destroy_tickets = 1;
+
 	if(k_hasafs())
-	    k_afsklog(0, 0);
+	    krb_afslog(0, 0);
 	reply(200, "Tickets will be destroyed on exit.");
 	return;
     }
@@ -219,13 +224,17 @@ void kauth(char *principal, char *ticket)
 			  pr.instance,
 			  pr.realm,
 			  KRB_TICKET_GRANTING_TICKET,
-			  pr.realm, 12,
+			  pr.realm,
+			  DEFAULT_TKT_LIFE,
 			  NULL, save_tkt, NULL);
     if(ret != INTK_BADPW){
 	reply(500, "Kerberos error: %s.", krb_get_err_text(ret));
 	return;
     }
-    base64_encode(cip.dat, cip.length, &p);
+    if(base64_encode(cip.dat, cip.length, &p) < 0) {
+	reply(500, "Out of memory while base64-encoding.");
+	return;
+    }
     reply(300, "P=%s T=%s", krb_unparse_name(&pr), p);
     free(p);
     memset(&cip, 0, sizeof(cip));
@@ -244,7 +253,8 @@ short_date(int32_t dp)
     return (cp);
 }
 
-void klist(void)
+void
+klist(void)
 {
     int err;
 
@@ -301,6 +311,8 @@ void klist(void)
      * it was done before tf_init.
      */
        
+    lreply(200, "Ticket file: %s", tkt_string());
+
     lreply(200, "Principal: %s", krb_unparse_name(&pr));
     while ((err = tf_get_cred(&c)) == KSUCCESS) {
 	if (header) {
@@ -322,3 +334,49 @@ void klist(void)
     }
     reply(200, "");
 }
+
+/*
+ * Only destroy if we created the tickets
+ */
+
+void
+cond_kdestroy(void)
+{
+    if (do_destroy_tickets)
+	dest_tkt();
+    afsunlog();
+}
+
+void
+kdestroy(void)
+{
+    dest_tkt();
+    afsunlog();
+    reply(200, "Tickets destroyed");
+}
+
+void
+krbtkfile(const char *tkfile)
+{
+    do_destroy_tickets = 0;
+    krb_set_tkt_string(tkfile);
+    reply(200, "Using ticket file %s", tkfile);
+}
+
+void
+afslog(const char *cell)
+{
+    if(k_hasafs()) {
+	krb_afslog(cell, 0);
+	reply(200, "afslog done");
+    } else {
+	reply(200, "no AFS present");
+    }
+}
+
+void
+afsunlog(void)
+{
+    if(k_hasafs())
+	k_unlog();
+}
