@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 1995, 1996 Kungliga Tekniska Hgskolan (Royal Institute
- * of Technology, Stockholm, Sweden).
+ * Copyright (c) 1995, 1996, 1997 Kungliga Tekniska Hgskolan
+ * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
  * Redistribution and use in source and binary forms, with or without
@@ -38,7 +38,7 @@
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
-RCSID("$Id: krb4.c,v 1.13 1996/10/27 23:51:18 assar Exp $");
+RCSID("$Id: krb4.c,v 1.23 1997/12/09 00:58:56 joda Exp $");
 #endif
 
 #ifdef HAVE_SYS_TYPES_H
@@ -86,17 +86,18 @@ int krb4_adat(char *auth)
     char inst[INST_SZ];
 
     memset(&tkt, 0, sizeof(tkt));
-    tkt.length = base64_decode(auth, tkt.dat);
+    len = base64_decode(auth, tkt.dat);
 
-    if(tkt.length < 0){
+    if(len < 0){
 	reply(501, "Failed to decode base64 data.");
 	return -1;
     }
+    tkt.length = len;
 
-    k_getsockinst(0, inst);
+    k_getsockinst(0, inst, sizeof(inst));
     kerror = krb_rd_req(&tkt, "ftp", inst, 0, &auth_dat, "");
     if(kerror == RD_AP_UNDEC){
-	k_getsockinst(0, inst);
+	k_getsockinst(0, inst, sizeof(inst));
 	kerror = krb_rd_req(&tkt, "rcmd", inst, 0, &auth_dat, "");
     }
 
@@ -121,7 +122,10 @@ int krb4_adat(char *auth)
 	reply(535, "Error creating reply: %s.", strerror(errno));
 	return -1;
     }
-    base64_encode(msg, len, &p);
+    if(base64_encode(msg, len, &p) < 0) {
+	reply(535, "Out of memory base64-encoding.");
+	return -1;
+    }
     reply(235, "ADAT=%s", p);
     auth_complete = 1;
     free(p);
@@ -157,8 +161,13 @@ int krb4_mic(char *msg)
     MSG_DAT m_data;
     char *tmp, *cmd;
   
-    cmd = strdup(msg);
+    cmd = malloc(strlen(msg) + 1);
     
+    if (cmd == NULL) {
+	reply(451, "Failed to allocate memory.");
+	return -1;
+    }
+
     len = base64_decode(msg, cmd);
     if(len < 0){
 	reply(501, "Failed to decode base 64 data.");
@@ -168,18 +177,22 @@ int krb4_mic(char *msg)
     kerror = krb_rd_safe(cmd, len, &auth_dat.session, 
 			 &his_addr, &ctrl_addr, &m_data);
 
+    free(cmd);
     if(kerror){
 	reply(535, "Error reading request: %s.", krb_get_err_text(kerror));
-	free(cmd);
 	return -1;
     }
     
-    tmp = strdup(msg);
-    sprintf(tmp, "%.*s", (int)m_data.app_length, m_data.app_data);
+    tmp = malloc(strlen(msg) + 1);
+    if (tmp == NULL) {
+	reply(451, "Failed to allocate memory.");
+	return -1;
+    }
+    snprintf(tmp, strlen(msg) + 1, "%.*s",
+	     (int)m_data.app_length, m_data.app_data);
     if(!strstr(tmp, "\r\n"))
 	strcat(tmp, "\r\n");
     new_ftp_command(tmp);
-    free(cmd);
     return 0;
 }
 
@@ -198,8 +211,8 @@ int krb4_enc(char *msg)
     MSG_DAT m_data;
     char *tmp, *cmd;
   
-    cmd = strdup(msg);
-    
+    cmd = malloc(strlen(msg) + 1);
+
     len = base64_decode(msg, cmd);
     if(len < 0){
 	reply(501, "Failed to decode base 64 data.");
@@ -215,8 +228,13 @@ int krb4_enc(char *msg)
 	return -1;
     }
     
-    tmp = strdup(msg);
-    sprintf(tmp, "%.*s", (int)m_data.app_length, m_data.app_data);
+    tmp = malloc(strlen(msg) + 1);
+    if (tmp == NULL) {
+	reply(451, "Failed to allocate memory.");
+	free(cmd);
+	return -1;
+    }
+    snprintf(tmp, strlen(msg) + 1, "%.*s", (int)m_data.app_length, m_data.app_data);
     if(!strstr(tmp, "\r\n"))
 	strcat(tmp, "\r\n");
     new_ftp_command(tmp);
@@ -320,7 +338,8 @@ int krb4_write(int fd, void *data, int length)
 	data_buffer[1] = (bytes >> 16) & 0xff;
 	data_buffer[2] = (bytes >> 8) & 0xff;
 	data_buffer[3] = bytes & 0xff;
-	krb_net_write(fd, data_buffer, bytes+4);
+	if(krb_net_write(fd, data_buffer, bytes+4) < 0)
+	    return -1;
 	length -= len;
 	data = (unsigned char*)data + len;
 	tx += len;
@@ -339,15 +358,16 @@ int krb4_userok(char *name)
 }
 
 
-int krb4_vprintf(const char *fmt, va_list ap)
+int
+krb4_vprintf(const char *fmt, va_list ap)
 {
-    char buf[10240]; /* XXX */
+    char buf[10240];
     char *p;
     char *enc;
     int code;
     int len;
   
-    vsprintf(buf, fmt, ap);
+    vsnprintf (buf, sizeof(buf), fmt, ap);
     enc = malloc(strlen(buf) + 31);
     if(prot_level == prot_safe){
 	len = krb_mk_safe((u_char*)buf, (u_char*)enc, strlen(buf), &auth_dat.session, 
@@ -361,9 +381,12 @@ int krb4_vprintf(const char *fmt, va_list ap)
 	len = 0; /* XXX */
 	code = 631;
     }
-    base64_encode(enc, len, &p);
-    fprintf(stdout, "%d %s\r\n", code, p);
+    if(base64_encode(enc, len, &p) < 0) {
+	fprintf(stdout, "451 base64-encode failed\r\n");
+    } else {
+	fprintf(stdout, "%d %s\r\n", code, p);
+	free(p);
+    }
     free(enc);
-    free(p);
     return 0;
 }
