@@ -46,7 +46,7 @@
 
 #include "kauth.h"
 
-RCSID("$Id: kauth.c,v 1.78 1997/11/06 20:57:35 joda Exp $");
+RCSID("$Id: kauth.c,v 1.81 1997/12/09 10:36:33 joda Exp $");
 
 krb_principal princ;
 static char srvtab[MaxPathLen + 1];
@@ -110,7 +110,7 @@ renew(int sig)
 	warnx ("%s", krb_get_err_text(code));
     else if (k_hasafs())
 	{
-	    if ((code = k_afsklog(cell, NULL)) != 0 && code != KDC_PR_UNKNOWN) {
+	    if ((code = krb_afslog(cell, NULL)) != 0 && code != KDC_PR_UNKNOWN) {
 		warnx ("%s", krb_get_err_text(code));
 	    }
 	}
@@ -278,10 +278,10 @@ main(int argc, char **argv)
 	    memset(passwd, 0, sizeof(passwd));
 	    exit(1);
 	}
-	code = krb_get_pw_in_tkt(princ.name, princ.instance, princ.realm, 
-				 KRB_TICKET_GRANTING_TICKET, princ.realm, 
-				 lifetime, passwd);
-
+	code = krb_get_pw_in_tkt2(princ.name, princ.instance, princ.realm, 
+				  KRB_TICKET_GRANTING_TICKET, princ.realm, 
+				  lifetime, passwd, &key);
+	
 	memset(passwd, 0, sizeof(passwd));
     }
     if (code) {
@@ -292,7 +292,7 @@ main(int argc, char **argv)
     if (k_hasafs()) {
 	if (more_args)
 	    k_setpag();
-	if ((code = k_afsklog(cell, NULL)) != 0 && code != KDC_PR_UNKNOWN)
+	if ((code = krb_afslog(cell, NULL)) != 0 && code != KDC_PR_UNKNOWN)
 	    warnx ("%s", krb_get_err_text(code));
     }
 
