@@ -33,7 +33,7 @@
 
 #include "telnet_locl.h"
 
-RCSID("$Id: commands.c,v 1.31 1997/05/13 10:20:54 joda Exp $");
+RCSID("$Id: commands.c,v 1.41 1997/10/29 01:26:34 assar Exp $");
 
 #if	defined(IPPROTO_IP) && defined(IP_TOS)
 int tos = -1;
@@ -1554,12 +1554,13 @@ env_init(void)
 		char hbuf[256+1];
 		char *cp2 = strchr((char *)ep->value, ':');
 
-		k_gethostname(hbuf, 256);
+		/* XXX - should be k_gethostname? */
+		gethostname(hbuf, 256);
 		hbuf[256] = '\0';
 
 		/* If this is not the full name, try to get it via DNS */
 		if (strchr(hbuf, '.') == 0) {
-			struct hostent *he = gethostbyname(hbuf);
+			struct hostent *he = roken_gethostbyname(hbuf);
 			if (he != 0)
 				strncpy(hbuf, he->h_name, 256);
 			hbuf[256] = '\0';
@@ -1601,7 +1602,7 @@ env_define(unsigned char *var, unsigned char *value)
 		if (ep->next)
 			ep->next->prev = ep;
 	}
-	ep->welldefined = opt_welldefined(var);
+	ep->welldefined = opt_welldefined((char *)var);
 	ep->export = 1;
 	ep->var = (unsigned char *)strdup((char *)var);
 	ep->value = (unsigned char *)strdup((char *)value);
@@ -1944,14 +1945,12 @@ ayt_status(void)
 }
 #endif
 
-static char *rcname = 0;
-static char rcbuf[128];
-
 static Command *getcmd(char *name);
 
 static void
 cmdrc(char *m1, char *m2)
 {
+    static char rcname[128];
     Command *c;
     FILE *rcfile;
     int gotmachine = 0;
@@ -1965,14 +1964,11 @@ cmdrc(char *m1, char *m2)
     strcpy(m1save, m1);
     m1 = m1save;
 
-    if (rcname == 0) {
-	rcname = getenv("HOME");
-	if (rcname)
-	    strcpy(rcbuf, rcname);
-	else
-	    rcbuf[0] = '\0';
-	strcat(rcbuf, "/.telnetrc");
-	rcname = rcbuf;
+    if (rcname[0] == 0) {
+	char *home = getenv("HOME");
+
+	snprintf (rcname, sizeof(rcname), "%s/.telnetrc",
+		  home ? home : "");
     }
 
     if ((rcfile = fopen(rcname, "r")) == 0) {
@@ -2033,7 +2029,12 @@ int
 tn(int argc, char **argv)
 {
     struct hostent *host = 0;
+#if defined(AF_INET6) && defined(HAVE_STRUCT_SOCKADDR_IN6)
+    struct sockaddr_in6 sin6;
+#endif
     struct sockaddr_in sin;
+    struct sockaddr *sa;
+    int sa_size;
     struct servent *sp = 0;
     unsigned long temp;
     extern char *inet_ntoa();
@@ -2042,9 +2043,9 @@ tn(int argc, char **argv)
     int srlen;
 #endif
     char *cmd, *hostp = 0, *portp = 0, *user = 0;
+    int family, port;
 
     /* clear the socket address prior to use */
-    memset(&sin, 0, sizeof(sin));
 
     if (connected) {
 	printf("?Already connected to %s\r\n", hostname);
@@ -2068,7 +2069,7 @@ tn(int argc, char **argv)
 	    --argc; ++argv;
 	    if (argc == 0)
 		goto usage;
-	    user = *argv++;
+	    user = strdup(*argv++);
 	    --argc;
 	    continue;
 	}
@@ -2115,27 +2116,58 @@ tn(int argc, char **argv)
 	    setuid(getuid());
 	    return 0;
 	} else {
-	    sin.sin_addr.s_addr = temp;
-	    sin.sin_family = AF_INET;
+	    abort();
 	}
     } else {
+#endif
+	memset (&sin, 0, sizeof(sin));
+#if defined(HAVE_INET_PTON) && defined(AF_INET6) && defined(HAVE_STRUCT_SOCKADDR_IN6)
+	memset (&sin6, 0, sizeof(sin6));
+
+	if(inet_pton(AF_INET6, hostp, &sin6.sin6_addr)) {
+	    sin6.sin6_family = family = AF_INET6;
+	    strcpy(_hostname, hostp);
+	    hostname =_hostname;
+	} else
 #endif
 	if(inet_aton(hostp, &sin.sin_addr)){
-	    sin.sin_family = AF_INET;
+	    sin.sin_family = family = AF_INET;
 	    strcpy(_hostname, hostp);
 	    hostname = _hostname;
 	} else {
-	    host = gethostbyname(hostp);
+#ifdef HAVE_GETHOSTBYNAME2
+	    host = gethostbyname2(hostp, AF_INET6);
+	    if(host == NULL)
+		host = gethostbyname2(hostp, AF_INET);
+#else
+	    host = roken_gethostbyname(hostp);
+#endif
 	    if (host) {
-		sin.sin_family = host->h_addrtype;
-#if	defined(h_addr)		/* In 4.3, this is a #define */
-		memmove(&sin.sin_addr,
-			host->h_addr_list[0],
-			sizeof(sin.sin_addr));
-#else	/* defined(h_addr) */
-		memmove(&sin.sin_addr, host->h_addr, sizeof(sin.sin_addr));
-#endif	/* defined(h_addr) */
 		strncpy(_hostname, host->h_name, sizeof(_hostname));
+		family = host->h_addrtype;
+
+		switch(family) {
+		case AF_INET:
+		    memset(&sin, 0, sizeof(sin));
+		    sa_size = sizeof(sin);
+		    sa = (struct sockaddr *)&sin;
+		    sin.sin_family = family;
+		    sin.sin_addr   = *((struct in_addr *)(*host->h_addr_list));
+		    break;
+#if defined(AF_INET6) && defined(HAVE_STRUCT_SOCKADDR_IN6)
+		case AF_INET6:
+		    memset(&sin6, 0, sizeof(sin6));
+		    sa_size = sizeof(sin6);
+		    sa = (struct sockaddr *)&sin6;
+		    sin6.sin6_family = family;
+		    sin6.sin6_addr   = *((struct in6_addr *)(*host->h_addr_list));
+		    break;
+#endif
+		default:
+		    fprintf(stderr, "Bad address family: %d\n", family);
+		    return 0;
+		}
+
 		_hostname[sizeof(_hostname)-1] = '\0';
 		hostname = _hostname;
 	    } else {
@@ -2158,34 +2190,60 @@ tn(int argc, char **argv)
 	    telnetport = 1;
 	} else
 	    telnetport = 0;
-	sin.sin_port = atoi(portp);
-	if (sin.sin_port == 0) {
-	    sp = getservbyname(portp, "tcp");
+	port = atoi(portp);
+	if (port == 0) {
+	    sp = roken_getservbyname(portp, "tcp");
 	    if (sp)
-		sin.sin_port = sp->s_port;
+		port = sp->s_port;
 	    else {
 		printf("%s: bad port number\r\n", portp);
 		setuid(getuid());
 		return 0;
 	    }
 	} else {
-	    sin.sin_port = htons(sin.sin_port);
+	    port = htons(port);
 	}
     } else {
 	if (sp == 0) {
-	    sp = getservbyname("telnet", "tcp");
+	    sp = roken_getservbyname("telnet", "tcp");
 	    if (sp == 0) {
 		fprintf(stderr, "telnet: tcp/telnet: unknown service\r\n");
 		setuid(getuid());
 		return 0;
 	    }
-	    sin.sin_port = sp->s_port;
+	    port = sp->s_port;
 	}
 	telnetport = 1;
     }
-    printf("Trying %s...\r\n", inet_ntoa(sin.sin_addr));
+    switch(family) {
+    case AF_INET:
+	sin.sin_port = port;
+	printf("Trying %s...\r\n", inet_ntoa(sin.sin_addr));
+	break;
+#if defined(AF_INET6) && defined(HAVE_STRUCT_SOCKADDR_IN6)
+    case AF_INET6: {
+#ifndef INET6_ADDRSTRLEN
+#define INET6_ADDRSTRLEN 46 
+#endif
+
+	char buf[INET6_ADDRSTRLEN];
+
+	sin6.sin6_port = port;
+#ifdef HAVE_INET_NTOP
+	printf("Trying %s...\r\n", inet_ntop(AF_INET6,
+					     &sin6.sin6_addr,
+					     buf,
+					     sizeof(buf)));
+#endif
+	break;
+    }
+#endif
+    default:
+	abort();
+    }
+
     do {
-	net = socket(AF_INET, SOCK_STREAM, 0);
+	net = socket(family, SOCK_STREAM, 0);
 	setuid(getuid());
 	if (net < 0) {
 	    perror("telnet: socket");
@@ -2217,23 +2275,36 @@ tn(int argc, char **argv)
 		perror("setsockopt (SO_DEBUG)");
 	}
 
-	if (connect(net, (struct sockaddr *)&sin, sizeof (sin)) < 0) {
-#if	defined(h_addr)		/* In 4.3, this is a #define */
+	if (connect(net, sa, sa_size) < 0) {
 	    if (host && host->h_addr_list[1]) {
 		int oerrno = errno;
 
-		fprintf(stderr, "telnet: connect to address %s: ",
-						inet_ntoa(sin.sin_addr));
+		switch(family) {
+		case AF_INET :
+		    fprintf(stderr, "telnet: connect to address %s: ",
+			    inet_ntoa(sin.sin_addr));
+		    sin.sin_addr = *((struct in_addr *)(*++host->h_addr_list));
+		    break;
+#if defined(AF_INET6) && defined(HAVE_STRUCT_SOCKADDR_IN6)
+		case AF_INET6: {
+		    char buf[INET6_ADDRSTRLEN];
+
+		    fprintf(stderr, "telnet: connect to address %s: ",
+			    inet_ntop(AF_INET6, &sin6.sin6_addr, buf,
+				      sizeof(buf)));
+		    sin6.sin6_addr = *((struct in6_addr *)(*++host->h_addr_list));
+		    break;
+		}
+#endif
+		default:
+		    abort();
+		}
+		    
 		errno = oerrno;
 		perror(NULL);
-		host->h_addr_list++;
-		memmove(&sin.sin_addr,
-			host->h_addr_list[0],
-			sizeof(sin.sin_addr));
 		NetClose(net);
 		continue;
 	    }
-#endif	/* defined(h_addr) */
 	    perror("telnet: Unable to connect to remote host");
 	    return 0;
 	}
@@ -2578,7 +2649,7 @@ sourceroute(char *arg, char **cpp, int *lenp)
 
 		if ((tmp = inet_addr(cp)) != -1) {
 			sin_addr.s_addr = tmp;
-		} else if ((host = gethostbyname(cp))) {
+		} else if ((host = roken_gethostbyname(cp))) {
 #if	defined(h_addr)
 			memmove(&sin_addr,
 				host->h_addr_list[0],
