@@ -8,8 +8,7 @@
  */
 #ifdef HAVE_CONFIG_H
 #include <config.h>
-#include <protos.h>
-RCSID("$Id: xnlock.c,v 1.58 1997/05/04 04:17:44 assar Exp $");
+RCSID("$Id: xnlock.c,v 1.64 1997/11/29 03:58:13 joda Exp $");
 #endif
 
 #include <stdio.h>
@@ -29,15 +28,30 @@ RCSID("$Id: xnlock.c,v 1.58 1997/05/04 04:17:44 assar Exp $");
 #include <pwd.h>
 #endif
 
+#ifdef KRB5
+#include <krb5.h>
+#endif
+#ifdef KRB4
 #include <krb.h>
+#endif
+#if defined(KRB4)
 #include <kafs.h>
+#endif
 
 #include <roken.h>
 #include <err.h>
 
+static char login[16];
+static char userprompt[128];
+#ifdef KRB4
 static char name[ANAME_SZ];
 static char inst[INST_SZ];
 static char realm[REALM_SZ + 1];
+#endif
+#ifdef KRB5
+static krb5_context context;
+static krb5_principal client;
+#endif
 
 #define font_height(font)	  	(font->ascent + font->descent)
 
@@ -70,8 +84,8 @@ static char	       *words;
 static int		x, y;
 static Pixel		Black, White;
 static XFontStruct    *font;
-static char		root_cpass[16];
-static char		user_cpass[16];
+static char		root_cpass[128];
+static char		user_cpass[128];
 static int		time_left, prompt_x, prompt_y, time_x, time_y;
 static unsigned long	interval;
 static Pixmap		left0, left1, right0, right1, left_front,
@@ -96,6 +110,7 @@ struct appres_t {
     Boolean accept_root;
     char *text, *text_prog, *file, *logoutPasswd;
     Boolean no_screensaver;
+    Boolean destroytickets;
 } appres;
 
 static XtResource resources[] = {
@@ -129,6 +144,9 @@ static XtResource resources[] = {
     
     { "noScreenSaver", "NoScreenSaver", XtRBoolean, sizeof(Boolean),
       XtOffsetOf(struct appres_t,no_screensaver), XtRImmediate, (XtPointer)True },
+
+    { "destroyTickets", "DestroyTickets", XtRBoolean, sizeof(Boolean),
+      XtOffsetOf(struct appres_t,destroytickets), XtRImmediate, (XtPointer)True },
 };
 
 static XrmOptionDescRec options[] = {
@@ -141,6 +159,7 @@ static XrmOptionDescRec options[] = {
     { "-ar",  ".acceptRootPasswd", XrmoptionNoArg, "True" },
     { "-noar", ".acceptRootPasswd", XrmoptionNoArg, "False" },
     { "-nonoscreensaver", ".noScreenSaver", XrmoptionNoArg, "False" },
+    { "-nodestroytickets", ".destroyTickets", XrmoptionNoArg, "False" },
 };
 
 static char*
@@ -187,6 +206,7 @@ usage(void)
     fprintf(stderr, "-nar          don't accept root's passwd\n");
     fprintf(stderr, "-f [file]     message is read from file or ~/.msgfile\n");
     fprintf(stderr, "-prog program  text is gotten from executing `program'\n");
+    fprintf(stderr, "-nodestroytickets keep kerberos tickets\n");
     exit(1);
 }
 
@@ -217,18 +237,17 @@ init_words (int argc, char **argv)
 		    errx (1, "cannot allocate memory for message");
 	    }
 	} else {
-	    appres.text = strdup("");
+	    int j;
+	    int len = 1;
+	    for(j = i; argv[j]; j++)
+		len += strlen(argv[j]) + 1;
+	    appres.text = malloc(len);
 	    if (appres.text == NULL)
 		errx (1, "cannot allocate memory for message");
-	    while (argv[i]) {
-		int n = strlen (argv[i]);
-		char *tmp = realloc(appres.text,
-				    strlen(appres.text) + n + 2);
-		if (tmp == NULL)
-		    errx (1, "cannot allocate memory for message");
-		strcat (appres.text, argv[i]);
-		strcat (appres.text, " ");
-		++i;
+	    appres.text[0] = 0;
+	    for(; i < j; i++){
+		strcat(appres.text, argv[i]);
+		strcat(appres.text, " ");
 	    }
 	}
     }
@@ -421,7 +440,6 @@ move(XtPointer _p, XtIntervalId *_id)
 static void
 post_prompt_box(Window window)
 {
-    char s[64];
     int width = (Width / 3);
     int height = font_height(font) * 6;
     int box_x, box_y;
@@ -438,11 +456,6 @@ post_prompt_box(Window window)
     time_y = prompt_y = Height / 2;
     box_y = prompt_y - 3 * font_height(font);
 
-    snprintf (s, sizeof(s), "User: %s%s%s@%s", name,
-	      inst[0] ? "." : "",
-	      inst[0] ? inst : "",
-	      realm);
-
     /* erase current guy -- text message may still exist */
     XSetForeground(dpy, gc, Black);
     XFillRectangle(dpy, window, gc, x, y, 64, 64);
@@ -459,7 +472,8 @@ post_prompt_box(Window window)
     XDrawRectangle(dpy, window, gc, box_x+12, box_y+12, width-23, height-23);
 
     XDrawString(dpy, window, gc,
-	prompt_x, prompt_y-font_height(font), s, strlen(s));
+		prompt_x, prompt_y-font_height(font), 
+		userprompt, strlen(userprompt));
     XDrawString(dpy, window, gc, prompt_x, prompt_y, PROMPT, strlen(PROMPT));
     /* set background for copyplane and DrawImageString; need reverse video */
     XSetBackground(dpy, gc, White);
@@ -533,6 +547,34 @@ countdown(XtPointer _t, XtIntervalId *_d)
     return;
 }
 
+#ifdef KRB5
+static int
+verify_krb5(const char *password)
+{
+    krb5_error_code ret;
+    krb5_ccache id;
+    
+    krb5_cc_default(context, &id);
+    ret = krb5_verify_user(context,
+			   client, 
+			   id,
+			   password, 
+			   0,
+			   NULL);
+    if (ret == 0){
+#ifdef KRB4
+	if (k_hasafs())
+	    krb5_afslog(context, id, NULL, NULL);
+#endif
+	return 0;
+    }
+    if (ret != KRB5KRB_AP_ERR_MODIFIED)
+	krb5_warn(context, ret, "verify_krb5");
+    
+    return -1;
+}
+#endif
+
 static int
 verify(char *password)
 {
@@ -560,22 +602,7 @@ verify(char *password)
 		 * continue */
 		signal(SIGHUP, SIG_DFL);
 	    }
-    /*
-     * Try to verify as user with kerberos.
-     */
 
-    ret = krb_verify_user(name, inst, realm, password, 0, NULL);
-    
-    if(ret == KSUCCESS){
-	if(k_hasafs())
-	    k_afsklog(0, 0);
-	return 0;
-    }
-
-    if(ret != INTK_BADPW)
-	warnx ("warning: %s",
-	       (ret < 0) ? strerror(ret) : krb_get_err_text(ret));
-    
     /*
      * Try copy of users password.
      */
@@ -585,8 +612,32 @@ verify(char *password)
     /*
      * Try to verify as user in case password change.
      */
-    if(unix_verify_user(name, password) == 0)
+    if (unix_verify_user(login, password) == 0)
 	return 0;
+
+#ifdef KRB5
+    /*
+     * Try to verify as user with kerberos 5.
+     */
+    if(verify_krb5(password) == 0)
+	return 0;
+#endif
+
+#ifdef KRB4
+    /*
+     * Try to verify as user with kerberos 4.
+     */
+    ret = krb_verify_user(name, inst, realm, password, 0, NULL);
+    if (ret == KSUCCESS){
+	if (k_hasafs())
+	    krb_afslog(NULL, NULL);
+	return 0;
+    }
+    if (ret != INTK_BADPW)
+	warnx ("warning: %s",
+	       (ret < 0) ? strerror(ret) : krb_get_err_text(ret));
+#endif
+    
     return -1;
 }
 
@@ -878,6 +929,8 @@ main (int argc, char **argv)
       strcpy(user_cpass, pw->pw_passwd);
       setuid(getuid());
       /* Now we're no longer running setuid root. */
+      strncpy(login, pw->pw_name, sizeof(login));
+      login[sizeof(login) - 1] = '\0';
     }
 
     srand(getpid());
@@ -886,8 +939,22 @@ main (int argc, char **argv)
 
     locked_at = time(0);
 
+    snprintf(userprompt, sizeof(userprompt), "User: %s", login);
+#ifdef KRB4
     krb_get_default_principal(name, inst, realm);
-    
+    snprintf(userprompt, sizeof(userprompt), "User: %s", 
+	     krb_unparse_name_long(name, inst, realm));
+#endif
+#ifdef KRB5
+    {
+	char *str;
+	krb5_init_context(&context);
+	krb5_get_default_principal(context, &client);
+	krb5_unparse_name(context, client, &str);
+	snprintf(userprompt, sizeof(userprompt), "User: %s", str);
+	free(str);
+    }
+#endif
 
     override = XtVaAppInitialize(&app, "XNlock", options, XtNumber(options),
 				 (Cardinal*)&argc, argv, NULL, 
@@ -901,6 +968,13 @@ main (int argc, char **argv)
     Black = appres.bg;
     White = appres.fg;
 
+    if (appres.destroytickets) {
+#ifdef KRB4
+        dest_tkt();		/* Nuke old ticket file */
+	creat(TKT_FILE, 0600);	/* but keep a place holder */
+#endif
+    }
+
     dpy = XtDisplay(override);
     
     if (dpy == 0)
@@ -993,3 +1067,4 @@ main (int argc, char **argv)
     XtAppMainLoop(app);
     exit(0);
 }
+
