@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 1995, 1996 Kungliga Tekniska Hgskolan (Royal Institute
- * of Technology, Stockholm, Sweden).
+ * Copyright (c) 1995, 1996, 1997 Kungliga Tekniska Hgskolan
+ * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
  * Redistribution and use in source and binary forms, with or without
@@ -39,7 +39,7 @@
 #include "kadm_locl.h"
 #include "ksrvutil.h"
 
-RCSID("$Id: ksrvutil_get.c,v 1.25 1996/11/24 00:59:48 assar Exp $");
+RCSID("$Id: ksrvutil_get.c,v 1.35 1997/12/05 04:27:02 assar Exp $");
 
 #define BAD_PW 1
 #define GOOD_PW 0
@@ -77,7 +77,7 @@ get_admin_password(char *myname, char *myinst, char *myrealm)
   CREDENTIALS c;
 
   if (princ_exists(myname, myinst, myrealm) != PE_NO) {
-    sprintf(buf, "Password for %s",
+    snprintf(buf, sizeof(buf), "Password for %s: ",
 	    krb_unparse_name_long (myname, myinst, myrealm));
     if (des_read_pw_string(admin_passwd, sizeof(admin_passwd)-1,
 			    buf, 0)) {
@@ -206,8 +206,8 @@ get_srvtab_ent(int fd, char *filename, char *name, char *inst, char *realm)
   if(ret == KADM_NOENTRY)
     ret = kadm_add(&values);
   if (ret != KSUCCESS) {
-    fprintf(stderr, "%s: Couldn't get srvtab entry for %s.%s: %s\n",
-	    progname, name, inst, error_message(ret));
+    warnx ("Couldn't get srvtab entry for %s.%s: %s",
+	    name, inst, error_message(ret));
     return;
   }
   
@@ -215,19 +215,29 @@ get_srvtab_ent(int fd, char *filename, char *name, char *inst, char *realm)
 
   /* get the key version number */
 
-  strcpy(old_tktfile, tkt_string());
-  sprintf(new_tktfile, TKT_ROOT "_ksrvutil-get.%d", (int)getpid());
-  krb_set_tkt_string(new_tktfile);
-
-  ret = krb_get_in_tkt(name, inst, realm, name, inst,
-		       1, key_to_key, NULL, &newkey);
-
+  { 
+      int old = krb_use_admin_server(1);
+      strcpy(old_tktfile, tkt_string());
+      snprintf(new_tktfile, sizeof(new_tktfile),
+	       TKT_ROOT "_ksrvutil-get.%u",
+	       (unsigned)getpid());
+      krb_set_tkt_string(new_tktfile);
+      
+      ret = krb_get_in_tkt(name, inst, realm, name, inst,
+			   1, key_to_key, NULL, &newkey);
+      krb_use_admin_server(old);
+  }
+      
   if (ret == KSUCCESS &&
       (ret = tf_init(tkt_string(), R_TKT_FIL)) == KSUCCESS &&
       (ret = tf_get_pname(garbage_name)) == KSUCCESS &&
       (ret = tf_get_pinst(garbage_inst)) == KSUCCESS &&
       (ret = tf_get_cred(&c)) == KSUCCESS)
-    kvno = c.kvno;
+      kvno = c.kvno;
+  else {
+      warnx ("Could not find the cred in the ticket file");
+      return;
+  }
 
   tf_close();
   krb_set_tkt_string(old_tktfile);
@@ -235,10 +245,9 @@ get_srvtab_ent(int fd, char *filename, char *name, char *inst, char *realm)
     
   if(ret != KSUCCESS) {
     memset(&newkey, 0, sizeof(newkey));
-    fprintf (stderr, "%s: Could not get a ticket for %s: %s\n",
-	     progname,
-	     krb_unparse_name_long(name, inst, realm),
-	     krb_get_err_text(ret));
+    warnx ("Could not get a ticket for %s: %s\n",
+	   krb_unparse_name_long(name, inst, realm),
+	   krb_get_err_text(ret));
     return;
   }
 
@@ -258,8 +267,7 @@ ksrvutil_kadm(int fd, char *filename, struct srv_ent *p)
   
   ret = kadm_init_link(PWSERV_NAME, KADM_SINST, u_realm);
   if (ret != KADM_SUCCESS) {
-    fprintf(stderr, "%s: Couldn't initialize kadmin link: %s\n",
-	    progname, error_message(ret));
+    warnx("Couldn't initialize kadmin link: %s", error_message(ret));
     leave(NULL, 1);
   }
   
@@ -272,14 +280,13 @@ ksrvutil_kadm(int fd, char *filename, struct srv_ent *p)
     /*
      *  create ticket file and get admin tickets
      */
-    sprintf(tktstring, TKT_ROOT "_ksrvutil_%d", (int)getpid());
+    snprintf(tktstring, sizeof(tktstring), TKT_ROOT "_ksrvutil_%d", (int)getpid());
     krb_set_tkt_string(tktstring);
     destroyp = TRUE;
        
     ret = get_admin_password(u_name, u_inst, u_realm);
     if (ret) {
-      fprintf(stderr, "%s: Couldn't get admin password.\n",
-	      progname);
+      warnx("Couldn't get admin password.");
       leave(NULL, 1);
     }
   }  
@@ -331,7 +338,7 @@ ksrvutil_get(int fd, char *filename, int argc, char **argv)
   struct srv_ent *head=NULL;
   int i;
 
-  k_gethostname(local_hostname, sizeof(local_hostname));
+  gethostname(local_hostname, sizeof(local_hostname));
   strcpy(local_hostname, krb_get_phost(local_hostname));
 
   if (argc)
@@ -339,14 +346,14 @@ ksrvutil_get(int fd, char *filename, int argc, char **argv)
       struct srv_ent *p=malloc(sizeof(*p));
 
       if(p == NULL) {
-	fprintf(stderr, "%s: out of memory in malloc\n", progname);
+	warnx ("out of memory in malloc");
 	leave(NULL,1);
       }
       p->next = head;
+      strcpy (p->realm, u_realm);
       if (kname_parse (p->name, p->inst, p->realm, argv[i]) !=
 	  KSUCCESS) {
-	fprintf (stderr, "%s: parse error on '%s'\n", progname,
-		 argv[i]);
+	warnx ("parse error on '%s'\n", argv[i]);
 	free(p);
 	continue;
       }
@@ -364,16 +371,20 @@ ksrvutil_get(int fd, char *filename, int argc, char **argv)
       safe_read_stdin("Name [rcmd]: ", databuf, sizeof(databuf));
       parseinput (sname, sizeof(sname), databuf, "rcmd");
     
-      sprintf(prompt, "Instance [%s]: ", local_hostname);
+      snprintf(prompt, sizeof(prompt), "Instance [%s]: ", local_hostname);
       safe_read_stdin(prompt, databuf, sizeof(databuf));
       parseinput (sinst, sizeof(sinst), databuf, local_hostname);
     
-      sprintf(prompt, "Realm [%s]: ", u_realm);
+      snprintf(prompt, sizeof(prompt), "Realm [%s]: ", u_realm);
       safe_read_stdin(prompt, databuf, sizeof(databuf));
       parseinput (srealm, sizeof(srealm), databuf, u_realm);
 
       if(yn("Is this correct?")){
 	struct srv_ent *p=(struct srv_ent*)malloc(sizeof(struct srv_ent));
+	if (p == NULL) {
+	    warnx ("out of memory in malloc");
+	    leave(NULL,1);
+	}
 	p->next=head;
 	head=p;
 	strcpy(p->name, sname);
