@@ -19,7 +19,7 @@
 
 #include "kuser_locl.h"
 
-RCSID("$Id: kinit.c,v 1.15 1997/03/30 18:58:46 assar Exp $");
+RCSID("$Id: kinit.c,v 1.16 1997/12/05 01:27:46 assar Exp $");
 
 #define	LIFE	DEFAULT_TKT_LIFE /* lifetime of ticket in 5-minute units */
 #define CHPASSLIFE 2
@@ -97,8 +97,8 @@ main(int argc, char **argv)
 	iflag = rflag = 1;
 	username = NULL;
     }
-    if (k_gethostname(buf, MaxHostNameLen)) 
-	err(1, "k_gethostname failed");
+    if (gethostname(buf, MaxHostNameLen)) 
+	err(1, "gethostname failed");
     printf("%s (%s)\n", ORGANIZATION, buf);
     if (username) {
 	printf("Kerberos Initialization for \"%s", aname);
