@@ -42,7 +42,7 @@
 
 #ifdef HAVE_CONFIG_H
 #include<config.h>
-RCSID("$Id: pam.c,v 1.9 1997/05/02 14:55:22 assar Exp $");
+RCSID("$Id: pam.c,v 1.11 1997/12/09 10:36:35 joda Exp $");
 #endif
 
 #include <stdio.h>
@@ -54,6 +54,7 @@ RCSID("$Id: pam.c,v 1.9 1997/05/02 14:55:22 assar Exp $");
 
 #define PAM_SM_AUTH
 #define PAM_SM_SESSION
+#include <security/pam_appl.h>
 #include <security/pam_modules.h>
 
 #include <netinet/in.h>
@@ -216,7 +217,7 @@ pam_sm_open_session(pam_handle_t *pamh, int flags, int argc, const char **argv)
     setenv("KRBTKFILE", tkt, 1);
     if(k_hasafs()){
 	k_setpag();
-	k_afsklog(0, 0);
+	krb_afslog(0, 0);
     }
     return PAM_SUCCESS;
 }
