@@ -1,8 +1,3 @@
-/*
- * This software may now be redistributed outside the US.
- *
- */
-
 /* 
   Copyright (C) 1989 by the Massachusetts Institute of Technology
 
@@ -26,7 +21,7 @@ or implied warranty.
 
 #include "kdb_locl.h"
 
-RCSID("$Id: krb_dbm.c,v 1.24 1996/11/19 22:48:45 assar Exp $");
+RCSID("$Id: krb_dbm.c,v 1.28 1997/12/05 01:28:07 assar Exp $");
 
 #include <xdbm.h>
 
@@ -103,7 +98,7 @@ static int non_blocking = 0;
  * Utility routine: generate name of database file.
  */
 
-static char *gen_dbsuffix __P((char *db_name, char *sfx));
+static char *gen_dbsuffix (char *db_name, char *sfx);
 
 static char *
 gen_dbsuffix(char *db_name, char *sfx)
@@ -113,14 +108,12 @@ gen_dbsuffix(char *db_name, char *sfx)
     if (sfx == NULL)
 	sfx = ".ok";
 
-    dbsuffix = malloc (strlen(db_name) + strlen(sfx) + 1);
-    strcpy(dbsuffix, db_name);
-    strcat(dbsuffix, sfx);
+    asprintf (&dbsuffix, "%s%s", db_name, sfx);
     return dbsuffix;
 }
 
 static void
-decode_princ_key __P((datum *key, char *name, char *instance));
+decode_princ_key (datum *key, char *name, char *instance);
 
 static void
 decode_princ_key(datum *key, char *name, char *instance)
@@ -132,7 +125,7 @@ decode_princ_key(datum *key, char *name, char *instance)
 }
 
 static void
-encode_princ_contents __P((datum *contents, Principal *principal));
+encode_princ_contents (datum *contents, Principal *principal);
 
 static void
 encode_princ_contents(datum *contents, Principal *principal)
@@ -164,7 +157,7 @@ static int mylock = 0;
 static int inited = 0;
 
 static int
-kerb_dbl_init __P((void));
+kerb_dbl_init (void);
 
 static int
 kerb_dbl_init()
@@ -184,7 +177,7 @@ kerb_dbl_init()
 }
 
 static void
-kerb_dbl_fini __P((void));
+kerb_dbl_fini (void);
 
 static void
 kerb_dbl_fini()
@@ -196,7 +189,7 @@ kerb_dbl_fini()
 }
 
 static int
-kerb_dbl_lock __P((int mode));
+kerb_dbl_lock (int mode);
 
 static int
 kerb_dbl_lock(int mode)
@@ -225,13 +218,13 @@ kerb_dbl_lock(int mode)
     if (non_blocking)
 	flock_mode |= K_LOCK_NB;
     
-    if (k_flock(dblfd, flock_mode) < 0) 
+    if (flock(dblfd, flock_mode) < 0) 
 	return errno;
     mylock++;
     return 0;
 }
 
-static void kerb_dbl_unlock __P((void));
+static void kerb_dbl_unlock (void);
 
 static void
 kerb_dbl_unlock()
@@ -241,17 +234,17 @@ kerb_dbl_unlock()
 	fflush(stderr);
 	exit(1);
     }
-    if (k_flock(dblfd, K_LOCK_UN) < 0) {
+    if (flock(dblfd, K_LOCK_UN) < 0) {
 	fprintf(stderr, "Kerberos database lock error. (unlocking)\n");
 	fflush(stderr);
-	perror("k_flock");
+	perror("flock");
 	exit(1);
     }
     mylock = 0;
 }
 
 int
-kerb_db_set_lockmode __P((int mode));
+kerb_db_set_lockmode (int mode);
 
 int
 kerb_db_set_lockmode(int mode)
@@ -266,7 +259,7 @@ kerb_db_set_lockmode(int mode)
  */
 
 int
-kerb_db_init __P((void));
+kerb_db_init (void);
 
 int
 kerb_db_init()
@@ -281,7 +274,7 @@ kerb_db_init()
  */
 
 void
-kerb_db_fini __P((void));
+kerb_db_fini (void);
 
 void
 kerb_db_fini()
@@ -296,7 +289,7 @@ kerb_db_fini()
  */
 
 int
-kerb_db_set_name __P((char *name));
+kerb_db_set_name (char *name);
 
 int
 kerb_db_set_name(char *name)
@@ -319,7 +312,7 @@ kerb_db_set_name(char *name)
  */
 
 time_t
-kerb_get_db_age __P((void));
+kerb_get_db_age (void);
 
 time_t
 kerb_get_db_age()
@@ -348,7 +341,7 @@ kerb_get_db_age()
  */
 
 static time_t
-kerb_start_update __P((char *db_name));
+kerb_start_update (char *db_name);
 
 static time_t
 kerb_start_update(char *db_name)
@@ -365,7 +358,7 @@ kerb_start_update(char *db_name)
 }
 
 static int
-kerb_end_update __P((char *db_name, time_t age));
+kerb_end_update (char *db_name, time_t age);
 
 static int
 kerb_end_update(char *db_name, time_t age)
@@ -402,7 +395,7 @@ kerb_end_update(char *db_name, time_t age)
 }
 
 static time_t
-kerb_start_read __P((void));
+kerb_start_read (void);
 
 static time_t
 kerb_start_read()
@@ -410,7 +403,7 @@ kerb_start_read()
     return kerb_get_db_age();
 }
 
-static int kerb_end_read __P((time_t age));
+static int kerb_end_read (time_t age);
 
 static int
 kerb_end_read(time_t age)
@@ -710,7 +703,7 @@ kerb_db_update(long *db, Principal *principal, unsigned int max)
  */
 
 int
-kerb_db_put_principal __P((Principal *principal, unsigned int max));
+kerb_db_put_principal (Principal *principal, unsigned int max);
 
 int
 kerb_db_put_principal(Principal *principal,
@@ -731,7 +724,7 @@ kerb_db_put_principal(Principal *principal,
 }
 
 void
-kerb_db_get_stat __P((DB_stat *s));
+kerb_db_get_stat (DB_stat *s);
 
 void
 kerb_db_get_stat(DB_stat *s)
@@ -752,7 +745,7 @@ kerb_db_get_stat(DB_stat *s)
 }
 
 void
-kerb_db_put_stat __P((DB_stat *s));
+kerb_db_put_stat (DB_stat *s);
 
 void
 kerb_db_put_stat(DB_stat *s)
@@ -760,7 +753,7 @@ kerb_db_put_stat(DB_stat *s)
 }
 
 void
-delta_stat __P((DB_stat *a, DB_stat *b, DB_stat *c));
+delta_stat (DB_stat *a, DB_stat *b, DB_stat *c);
 
 void
 delta_stat(DB_stat *a, DB_stat *b, DB_stat *c)
@@ -788,7 +781,7 @@ delta_stat(DB_stat *a, DB_stat *b, DB_stat *c)
  */
 
 int
-kerb_db_get_dba __P((char *dba_name, char *dba_inst, Dba *dba, unsigned int max, int *more));
+kerb_db_get_dba (char *dba_name, char *dba_inst, Dba *dba, unsigned int max, int *more);
 
 int
 kerb_db_get_dba(char *dba_name, char *dba_inst, Dba *dba,
