@@ -38,7 +38,7 @@
 
 #include "krb_locl.h"
 
-RCSID("$Id: get_ad_tkt.c,v 1.15 1997/04/01 08:18:27 joda Exp $");
+RCSID("$Id: get_ad_tkt.c,v 1.17 1997/12/05 01:53:06 assar Exp $");
 
 /*
  * get_ad_tkt obtains a new service ticket from Kerberos, using
@@ -87,7 +87,19 @@ get_ad_tkt(char *service, char *sinstance, char *realm, int lifetime)
     int kerror;
     unsigned char *p;
 
-    if ((kerror = krb_get_tf_realm(TKT_FILE, lrealm)) != KSUCCESS)
+    /*
+     * First check if we have a "real" TGT for the corresponding
+     * realm, if we don't, use ordinary inter-realm authentication.
+     */
+
+    kerror = krb_get_cred(KRB_TICKET_GRANTING_TICKET, realm, realm, &cr);
+    if (kerror == KSUCCESS) {
+      strncpy(lrealm, realm, REALM_SZ);
+      lrealm[REALM_SZ - 1] = '\0';
+    } else
+      kerror = krb_get_tf_realm(TKT_FILE, lrealm);
+    
+    if (kerror != KSUCCESS)
 	return(kerror);
 
     /*
