@@ -38,7 +38,7 @@
 
 #include "krb_locl.h"
 
-RCSID("$Id: get_host.c,v 1.31 1997/09/26 17:42:37 joda Exp $");
+RCSID("$Id: get_host.c,v 1.34 1997/12/05 09:04:43 joda Exp $");
 
 static struct host_list {
     struct krb_host *this;
@@ -63,7 +63,8 @@ free_hosts(struct host_list *h)
 }
 
 static int
-parse_address(char *address, enum krb_host_proto *proto, char **host, int *port)
+parse_address(char *address, enum krb_host_proto *proto,
+	      char **host, int *port)
 {
     char *p, *q;
     int default_port = krb_port;
@@ -101,6 +102,8 @@ parse_address(char *address, enum krb_host_proto *proto, char **host, int *port)
     q = strchr(p, ':');
     if(q){
 	*host = (char*)malloc(q - p + 1);
+	if (*host == NULL)
+	    return -1;
 	strncpy(*host, p, q - p);
 	(*host)[q - p] = 0;
 	q++;
@@ -117,6 +120,8 @@ parse_address(char *address, enum krb_host_proto *proto, char **host, int *port)
 	}
     }else{
 	*host = strdup(p);
+	if(*host == NULL)
+	    return -1;
 	*port = default_port;
     }
     return 0;
@@ -127,8 +132,12 @@ add_host(char *realm, char *address, int admin, int validate)
 {
     struct krb_host *host;
     struct host_list *p, **last = &hosts;
+
     host = (struct krb_host*)malloc(sizeof(struct krb_host));
-    parse_address(address, &host->proto, &host->host, &host->port);
+    if (host == NULL)
+	return 1;
+    if(parse_address(address, &host->proto, &host->host, &host->port) < 0)
+	return 1;
     if(validate && gethostbyname(host->host) == NULL){
 	free(host->host);
 	free(host);
@@ -147,7 +156,18 @@ add_host(char *realm, char *address, int admin, int validate)
 	last = &p->next;
     }
     host->realm = strdup(realm);
+    if (host->realm == NULL) {
+	free(host->host);
+	free(host);
+	return 1;
+    }
     p = (struct host_list*)malloc(sizeof(struct host_list));
+    if (p == NULL) {
+	free(host->realm);
+	free(host->host);
+	free(host);
+	return 1;
+    }
     p->this = host;
     p->next = NULL;
     *last = p;
@@ -169,7 +189,7 @@ read_file(const char *filename, const char *r)
     FILE *f = fopen(filename, "r");
     if(f == NULL)
 	return -1;
-    while(fgets(line, sizeof(line), f)){
+    while(fgets(line, sizeof(line), f) != NULL) {
 	n = sscanf(line, "%s %s admin %s", realm, address, scratch);
 	if(n == 2 || n == 3){
 	    if(strcmp(realm, r))
@@ -185,18 +205,12 @@ read_file(const char *filename, const char *r)
 static int
 init_hosts(char *realm)
 {
-    static const char *files[] = KRB_CNF_FILES;
     int i;
-    char *dir = getenv("KRBCONFDIR");
-
+    char file[128];
+    
     krb_port = ntohs(k_getportbyname (KRB_SERVICE, NULL, htons(KRB_PORT)));
-    if(dir){
-	char file[MaxPathLen];
-	if(k_concat(file, sizeof(file), dir, "/krb.conf", NULL) == 0)
-	    read_file(file, realm);
-    }
-    for(i = 0; files[i]; i++)
-	read_file(files[i], realm);
+    for(i = 0; krb_get_krbconf(i, file, sizeof(file)) == 0; i++)
+	read_file(file, realm);
     return 0;
 }
 
@@ -207,7 +221,7 @@ srv_find_realm(char *realm, char *proto, char *service)
     struct dns_reply *r;
     struct resource_record *rr;
     
-    k_mconcat(&domain, 1024, service, ".", proto, ".", realm, ".", NULL);
+    roken_mconcat(&domain, 1024, service, ".", proto, ".", realm, ".", NULL);
     
     if(domain == NULL)
 	return;
@@ -294,7 +308,8 @@ krb_get_krbhst(char *host, char *realm, int nth)
     struct krb_host *p = krb_get_host(nth, realm, 0);
     if(p == NULL)
 	return KFAILURE;
-    strcpy(host, p->host);
+    strncpy(host, p->host, MaxHostNameLen);
+    host[MaxHostNameLen - 1] = '\0';
     return KSUCCESS;
 }
 
@@ -304,6 +319,7 @@ krb_get_admhst(char *host, char *realm, int nth)
     struct krb_host *p = krb_get_host(nth, realm, 1);
     if(p == NULL)
 	return KFAILURE;
-    strcpy(host, p->host);
+    strncpy(host, p->host, MaxHostNameLen);
+    host[MaxHostNameLen - 1] = '\0';
     return KSUCCESS;
 }
