@@ -1,8 +1,3 @@
-/*
- * This software may now be redistributed outside the US.
- *
- */
-
 /* 
   Copyright (C) 1989 by the Massachusetts Institute of Technology
 
@@ -26,7 +21,7 @@ or implied warranty.
 
 #include "krb_locl.h"
 
-RCSID("$Id: get_krbrlm.c,v 1.10 1996/11/17 20:23:44 assar Exp $");
+RCSID("$Id: get_krbrlm.c,v 1.18 1997/12/05 09:04:58 joda Exp $");
 
 /*
  * krb_get_lrealm takes a pointer to a string, and a number, n.  It fills
@@ -46,72 +41,57 @@ RCSID("$Id: get_krbrlm.c,v 1.10 1996/11/17 20:23:44 assar Exp $");
 static int
 krb_get_lrealm_f(char *r, int n, const char *fname)
 {
-  FILE *cnffile;
-
-  if ((cnffile = fopen(fname, "r")) == NULL)
-    return(KFAILURE);
-
-  if (fscanf(cnffile,"%s",r) != 1)
-    {
-      fclose(cnffile);
-      return(KFAILURE);
+    FILE *f;
+    int ret = KFAILURE;
+    f = fopen(fname, "r");
+    if(f){
+	char buf[REALM_SZ];
+	if(fgets(buf, sizeof(buf), f)){
+	    char *p = buf + strspn(buf, " \t");
+	    p[strcspn(p, " \t\r\n")] = 0;
+	    p[REALM_SZ - 1] = 0;
+	    strcpy(r, p);
+	    ret = KSUCCESS;
+	}
+	fclose(f);
     }
-  fclose(cnffile);
-  return(KSUCCESS);
+    return ret;
 }
 
 int
 krb_get_lrealm(char *r, int n)
 {
-  static const char *const files[] = KRB_CNF_FILES;
-  int i;
-  
-  const char *dir = getenv("KRBCONFDIR");
+    int i;
+    char file[128];
 
-  if (n > 1)
-    return(KFAILURE);		/* Temporary restriction */
+    if (n > 1)
+	return(KFAILURE);		/* Temporary restriction */
 
-  /* First try user specified file */
-  if (dir != 0) {
-    char fname[MaxPathLen];
+    for (i = 0; krb_get_krbconf(i, file, sizeof(file)) == 0; i++)
+	if (krb_get_lrealm_f(r, n, file) == KSUCCESS)
+	    return KSUCCESS;
 
-    strncpy(fname, dir, sizeof(fname));
-    fname[sizeof(fname)-1] = '\0';
-    strncat(fname, "/krb.conf", sizeof(fname) - strlen(fname) - 1);
-    fname[sizeof(fname)-1] = '\0';
-    if (krb_get_lrealm_f(r, n, fname) == KSUCCESS)
-      return KSUCCESS;
-  }
+    /* If nothing else works try LOCALDOMAIN, if it exists */
+    if (n == 1) {
+	char *t, hostname[MaxHostNameLen];
+	gethostname(hostname, sizeof(hostname));
+	t = krb_realmofhost(hostname);
+	if (t) {
+	    strcpy (r, t);
+	    return KSUCCESS;
+	}
+	t = strchr(hostname, '.');
+	if (t == 0)
+	    return KFAILURE;	/* No domain part, you loose */
 
-  for (i = 0; files[i] != 0; i++)
-    if (krb_get_lrealm_f(r, n, files[i]) == KSUCCESS)
-      return KSUCCESS;
-
-  /* If nothing else works try LOCALDOMAIN, if it exists */
-  if (n == 1)
-    {
-      char *t, hostname[MaxHostNameLen];
-      k_gethostname(hostname, sizeof(hostname));
-      t = krb_realmofhost(hostname);
-      if (t) {
-	strcpy (r, t);
-	return KSUCCESS;
-      }
-      t = strchr(hostname, '.');
-      if (t == 0)
-	return KFAILURE;	/* No domain part, you loose */
-
-      t++;			/* Skip leading dot and upcase the rest */
-      for (; *t; t++, r++)
-	if (islower(*t))
-	  *r = toupper(*t);
-	else
-	  *r = *t;
-      *r = 0;
-      return(KSUCCESS);
+	t++;			/* Skip leading dot and upcase the rest */
+	for (; *t; t++, r++)
+	    *r = toupper(*t);
+	*r = 0;
+	return(KSUCCESS);
     }
-  else
-    return(KFAILURE);
+    else
+	return(KFAILURE);
 }
 
 /* For SunOS5 compat. */
