@@ -21,7 +21,7 @@ or implied warranty.
 
 #include "krb_locl.h"
 
-RCSID("$Id: getrealm.c,v 1.19 1996/11/17 20:23:45 assar Exp $");
+RCSID("$Id: getrealm.c,v 1.29 1997/12/05 09:05:12 joda Exp $");
 
 #define MATCH_SUBDOMAINS        0
 
@@ -57,25 +57,26 @@ RCSID("$Id: getrealm.c,v 1.19 1996/11/17 20:23:45 assar Exp $");
 static int
 dns_find_realm(char *hostname, char *realm)
 {
-    char domain[MaxHostNameLen];
-    char host[MaxHostNameLen];
+    char domain[MaxHostNameLen + sizeof("krb4-realm..")];
     char *p;
     int level = 0;
     struct dns_reply *r;
     
-    strcpy(host, hostname);
-    if(host[strlen(host) - 1] != '.')
-	strcat(host, ".");
-    p = host;
+    p = hostname;
 
     while(1){
-	sprintf(domain, "krb4-realm.%s.", p);
+	snprintf(domain, sizeof(domain), "krb4-realm.%s.", p);
+	p = strchr(p, '.');
+	if(p == NULL)
+	    break;
+	p++;
 	r = dns_lookup(domain, "TXT");
 	if(r){
 	    struct resource_record *rr = r->head;
 	    while(rr){
 		if(rr->type == T_TXT){
 		    strncpy(realm, rr->u.txt, REALM_SZ);
+		    realm[REALM_SZ - 1] = 0;
 		    dns_free_data(r);
 		    return level;
 		}
@@ -84,10 +85,6 @@ dns_find_realm(char *hostname, char *realm)
 	    dns_free_data(r);
 	}
 	level++;
-	p = strchr(p, '.');
-	if(p == NULL)
-	    break;
-	p++;
     }
     return -1;
 }
@@ -96,102 +93,92 @@ dns_find_realm(char *hostname, char *realm)
 static FILE *
 open_krb_realms(void)
 {
-  static const char *const files[] = KRB_RLM_FILES;
-  FILE *res;
-  int i;
-  
-  const char *dir = getenv("KRBCONFDIR");
-
-  /* First try user specified file */
-  if (dir != 0) {
-    char fname[MaxPathLen];
-
-    strncpy(fname, dir, sizeof(fname));
-    fname[sizeof(fname)-1] = '\0';
-    strncat(fname, "/krb.realms", sizeof(fname) - strlen(fname) - 1);
-    fname[sizeof(fname)-1] = '\0';
-    if ((res = fopen(fname, "r")) != NULL)
-      return res;
-  }
-
-  for (i = 0; files[i] != 0; i++)
-    if ((res = fopen(files[i], "r")) != NULL)
-      return res;
-
+    int i;
+    char file[128];
+    FILE *res;
+    for(i = 0; krb_get_krbrealms(i, file, sizeof(file)) == 0; i++)
+	if ((res = fopen(file, "r")) != NULL)
+	    return res;
   return NULL;
 }
 
+static int
+file_find_realm(const char *phost, const char *domain,
+		char *ret_realm, size_t ret_realm_sz)
+{
+    FILE *trans_file;
+    char buf[1024];
+    int ret = -1;
+    
+    if ((trans_file = open_krb_realms()) == NULL)
+	return -1;
+
+    while (fgets(buf, sizeof(buf), trans_file) != NULL) {
+	char *save = NULL;
+	char *tok;
+	char *tmp_host;
+	char *tmp_realm;
+
+	tok = strtok_r(buf, " \t\r\n", &save);
+	if(tok == NULL)
+	    continue;
+	tmp_host = tok;
+	tok = strtok_r(NULL, " \t\r\n", &save);
+	if(tok == NULL)
+	    continue;
+	tmp_realm = tok;
+	if (strcasecmp(tmp_host, phost) == 0) {
+	    /* exact match of hostname, so return the realm */
+	    strncpy(ret_realm, tmp_realm, ret_realm_sz);
+	    ret_realm[ret_realm_sz - 1] = '\0';
+	    ret = 0;
+	    break;
+	}
+	if ((tmp_host[0] == '.') && domain) { 
+	    const char *cp = domain;
+	    do {
+		if(strcasecmp(tmp_host, domain) == 0){
+		    /* domain match, save for later */ 
+		    strncpy(ret_realm, tmp_realm, ret_realm_sz);
+		    ret_realm[ret_realm_sz - 1] = '\0';
+		    ret = 0;
+		    break;
+		}
+		cp = strchr(cp + 1, '.');
+	    } while(MATCH_SUBDOMAINS && cp);
+	}
+    }
+    fclose(trans_file);
+    return ret;
+}
+
 char *
 krb_realmofhost(const char *host)
 {
-  static char ret_realm[REALM_SZ+1];
-  char *domain;
-  FILE *trans_file;
-  char trans_host[MaxHostNameLen+1];
-  char trans_realm[REALM_SZ+1];
-  int retval;
-
-  char phost[MaxHostNameLen+1];
+    static char ret_realm[REALM_SZ];
+    char *domain;
+    char phost[MaxHostNameLen];
 	
-  krb_name_to_name(host, phost);
+    krb_name_to_name(host, phost, sizeof(phost));
 	
-  domain = strchr(phost, '.');
+    domain = strchr(phost, '.');
 
-  /* prepare default */
-  if(dns_find_realm(phost, ret_realm) < 0){
+    if(file_find_realm(phost, domain, ret_realm, sizeof ret_realm) == 0)
+	return ret_realm;
+
+    if(dns_find_realm(phost, ret_realm) >= 0)
+	return ret_realm;
+  
     if (domain) {
-      char *cp;
-		
-      strncpy(ret_realm, &domain[1], REALM_SZ);
-      ret_realm[REALM_SZ] = '\0';
-      /* Upper-case realm */
-      for (cp = ret_realm; *cp; cp++)
-	*cp = toupper(*cp);
+	char *cp;
+	  
+	strncpy(ret_realm, &domain[1], REALM_SZ);
+	ret_realm[REALM_SZ - 1] = 0;
+	/* Upper-case realm */
+	for (cp = ret_realm; *cp; cp++)
+	    *cp = toupper(*cp);
     } else {
-      krb_get_lrealm(ret_realm, 1);
+	krb_get_lrealm(ret_realm, 1);
     }
-  }
-
-  if ((trans_file = open_krb_realms()) == NULL)
-    return(ret_realm); /* krb_errno = KRB_NO_TRANS */
-
-  while (1) {
-    if ((retval = fscanf(trans_file, "%s %s",
-			 trans_host, trans_realm)) != 2) {
-      if (retval == EOF) {
-	fclose(trans_file);
-	return(ret_realm);
-      }
-      continue;	/* ignore broken lines */
-    }
-    trans_host[MaxHostNameLen] = '\0';
-    trans_realm[REALM_SZ] = '\0';
-    if (!strcasecmp(trans_host, phost)) {
-      /* exact match of hostname, so return the realm */
-      strcpy(ret_realm, trans_realm);
-      fclose(trans_file);
-      return(ret_realm);
-    }
-    if ((trans_host[0] == '.') && domain) { 
-#if     MATCH_SUBDOMAINS
-      char *cp;
-      for (cp = domain; cp != NULL; cp = strchr(cp+1, '.')) {
-	/* this is a domain match */
-	if (!strcasecmp(trans_host, cp)) {
-	  /* domain match, save for later */
-	  strcpy(ret_realm, trans_realm);
-	  continue;
-	}
-      }
-#else   /* MATCH_SUBDOMAINS */
-      /* this is a domain match */ 
-      if (!strcasecmp(trans_host, domain)) { 
-	/* domain match, save for later */ 
-	strcpy(ret_realm, trans_realm); 
-	continue; 
-      } 
-#endif  /* MATCH_SUBDOMAINS */
-    }
-  }
-  return NULL;
+    return ret_realm;
 }
