@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 1995, 1996 Kungliga Tekniska Hgskolan (Royal Institute
- * of Technology, Stockholm, Sweden).
+ * Copyright (c) 1995, 1996, 1997 Kungliga Tekniska Hgskolan
+ * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
  * Redistribution and use in source and binary forms, with or without
@@ -38,7 +38,7 @@
 
 #include "krb_locl.h"
 
-RCSID("$Id: krb_get_in_tkt.c,v 1.17 1996/10/27 16:29:46 joda Exp $");
+RCSID("$Id: krb_get_in_tkt.c,v 1.23 1997/12/05 00:17:22 joda Exp $");
 
 /*
  * decrypt_tkt(): Given user, instance, realm, passwd, key_proc
@@ -105,15 +105,14 @@ decrypt_tkt(char *user, char *instance, char *realm,
  */
 
 int
-krb_get_in_tkt(char *user, char *instance, char *realm, 
-	       char *service, char *sinstance, int life,
-	       key_proc_t key_proc, decrypt_proc_t decrypt_proc, void *arg)
+krb_mk_as_req(char *user, char *instance, char *realm, 
+	      char *service, char *sinstance, int life, KTEXT cip)
 {
     KTEXT_ST pkt_st;
     KTEXT pkt = &pkt_st;	/* Packet to KDC */
     KTEXT_ST rpkt_st;
-    KTEXT rpkt = &rpkt_st;	/* Returned packet */
-
+    KTEXT rpkt = &rpkt_st;	/* Reply from KDC */
+    
     int kerror;
     struct timeval tv;
 
@@ -138,53 +137,59 @@ krb_get_in_tkt(char *user, char *instance, char *realm,
 
     /* SEND THE REQUEST AND RECEIVE THE RETURN PACKET */
 
-    if ((kerror = send_to_kdc(pkt, rpkt, realm))) return(kerror);
-    
-    p = rpkt->dat;
+    kerror = send_to_kdc(pkt, rpkt, realm);
+    if(kerror) return kerror;
+    kerror = kdc_reply_cipher(rpkt, cip);
+    return kerror;
+}
+
+int
+krb_decode_as_rep(char *user, char *instance, char *realm,
+		  char *service, char *sinstance, 
+		  key_proc_t key_proc, decrypt_proc_t decrypt_proc, void *arg,
+		  KTEXT as_rep, CREDENTIALS *cred)
+{
+    int kerror;
+    time_t now;
     
-    {
-	CREDENTIALS cred;
-	KTEXT_ST cip;
-	KTEXT foo = &cip; /* braindamage */
-	
-	kerror = kdc_reply_cipher(rpkt, &cip);
-	if(kerror != KSUCCESS)
-	    return kerror;
+    if (decrypt_proc == NULL)
+	decrypt_proc = decrypt_tkt;
+    (*decrypt_proc)(user, instance, realm, arg, key_proc, &as_rep);
 
-	if (decrypt_proc == NULL)
-	    decrypt_proc = decrypt_tkt;
-	(*decrypt_proc)(user, instance, realm, arg, key_proc, &foo);
-
-	kerror = kdc_reply_cred(&cip, &cred);
-	if(kerror != KSUCCESS)
-	    return kerror;
+    kerror = kdc_reply_cred(as_rep, cred);
+    if(kerror != KSUCCESS)
+	return kerror;
 	
-	if (strcmp(cred.service, service) || 
-	    strcmp(cred.instance, sinstance) ||
-	    strcmp(cred.realm, realm))	/* not what we asked for */
-	    return INTK_ERR;	/* we need a better code here XXX */
+    if (strcmp(cred->service, service) || 
+	strcmp(cred->instance, sinstance) ||
+	strcmp(cred->realm, realm))	/* not what we asked for */
+	return INTK_ERR;	/* we need a better code here XXX */
 
-	if (abs((int)(tv.tv_sec - cred.issue_date)) > CLOCK_SKEW) {
-	    return RD_AP_TIME; /* XXX should probably be better code */
-	}
+    now = time(NULL);
+    if (abs((int)(now - cred->issue_date)) > CLOCK_SKEW) {
+	return RD_AP_TIME; /* XXX should probably be better code */
+    }
 
-	/* initialize ticket cache */
+    return 0;
+}
 
-	kerror = tf_create(TKT_FILE);
-	if (kerror != KSUCCESS)
-	    return kerror;
+int
+krb_get_in_tkt(char *user, char *instance, char *realm, 
+	       char *service, char *sinstance, int life,
+	       key_proc_t key_proc, decrypt_proc_t decrypt_proc, void *arg)
+{
+    KTEXT_ST as_rep;
+    CREDENTIALS cred;
+    int ret;
 
-	if (tf_put_pname(user) != KSUCCESS ||
-	    tf_put_pinst(instance) != KSUCCESS) {
-	    tf_close();
-	    return INTK_ERR;
-	}
+    ret = krb_mk_as_req(user, instance, realm, 
+			service, sinstance, life, &as_rep);
+    if(ret)
+	return ret;
+    ret = krb_decode_as_rep(user, instance, realm, service, sinstance, 
+			    key_proc, decrypt_proc, arg, &as_rep, &cred);
+    if(ret)
+	return ret;
 
-	kerror = tf_save_cred(cred.service, cred.instance, cred.realm, 
-			      cred.session, cred.lifetime, cred.kvno,
-			      &cred.ticket_st, cred.issue_date);
-	tf_close();
-    
-	return kerror;
-    }
+    return tf_setup(&cred, user, instance);
 }
