@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 1995, 1996 Kungliga Tekniska Hgskolan (Royal Institute
- * of Technology, Stockholm, Sweden).
+ * Copyright (c) 1995, 1996, 1997 Kungliga Tekniska Hgskolan
+ * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
  * Redistribution and use in source and binary forms, with or without
@@ -38,10 +38,10 @@
 
 #include "krb_locl.h"
 
-RCSID("$Id: mk_priv.c,v 1.17 1996/10/27 16:29:49 joda Exp $");
+RCSID("$Id: mk_priv.c,v 1.19 1997/12/05 02:01:59 joda Exp $");
 
 /* application include files */
-#include "lsb_addr_comp.h"
+#include "krb-archaeology.h"
 
 /*
  * krb_mk_priv() constructs an AUTH_MSG_PRIVATE message.  It takes
