@@ -38,7 +38,7 @@
 
 #include "krb_locl.h"
 
-RCSID("$Id: mk_req.c,v 1.16 1997/04/01 08:18:38 joda Exp $");
+RCSID("$Id: mk_req.c,v 1.18 1997/12/05 01:55:27 assar Exp $");
 
 static int lifetime = 255;	/* But no longer than TGT says. */
 
@@ -133,32 +133,40 @@ krb_mk_req(KTEXT authent, char *service, char *instance, char *realm,
     /* Get the ticket and move it into the authenticator */
     if (krb_ap_req_debug)
         krb_warning("Realm: %s\n", realm);
-    /* 
-     * Determine realm of these tickets.  We will send this to the
-     * KDC from which we are requesting tickets so it knows what to
-     * with our session key.
-     */
 
-    retval = krb_get_tf_realm(TKT_FILE, myrealm);
-    
-    if (retval != KSUCCESS)
-	return retval;
-    
     retval = krb_get_cred(service,instance,realm,&cr);
 
     if (retval == RET_NOTKT) {
 	retval = get_ad_tkt(service, instance, realm, lifetime);
-	if(retval == KSUCCESS)
+	if (retval == KSUCCESS)
 	    retval = krb_get_cred(service, instance, realm, &cr);
     }
 
-    if(retval != KSUCCESS)
+    if (retval != KSUCCESS)
 	return retval;
 
 
+    /*
+     * With multi realm ticket files either find a matching TGT or
+     * else use the first TGT for inter-realm authentication.
+     *
+     * In myrealm hold the realm of the principal "owning" the
+     * corresponding ticket-granting-ticket.
+     */
+
+    retval = krb_get_cred(KRB_TICKET_GRANTING_TICKET, realm, realm, 0);
+    if (retval == KSUCCESS) {
+      strncpy(myrealm, realm, REALM_SZ);
+      myrealm[REALM_SZ - 1] = '\0';
+    } else
+      retval = krb_get_tf_realm(TKT_FILE, myrealm);
+    
+    if (retval != KSUCCESS)
+	return retval;
+    
     if (krb_ap_req_debug)
-        krb_warning("%s %s %s %s %s\n", service, instance, realm,
-		    cr.pname, cr.pinst);
+        krb_warning("serv=%s.%s@%s princ=%s.%s@%s\n", service, instance, realm,
+		    cr.pname, cr.pinst, myrealm);
 
     p += krb_put_int(cr.kvno, p, 1);
 
