@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 1995, 1996 Kungliga Tekniska Hgskolan (Royal Institute
- * of Technology, Stockholm, Sweden).
+ * Copyright (c) 1995, 1996, 1997 Kungliga Tekniska Hgskolan
+ * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
  * Redistribution and use in source and binary forms, with or without
@@ -38,12 +38,15 @@
 
 #include "krb_locl.h"
 
-RCSID("$Id: mk_safe.c,v 1.16 1996/10/27 16:29:50 joda Exp $");
+RCSID("$Id: mk_safe.c,v 1.22 1997/12/05 02:01:52 joda Exp $");
 
 /* application include files */
-#include "lsb_addr_comp.h"
+#include "krb-archaeology.h"
 
 
+/* from rd_safe.c */
+extern int dqc_type;
+void fixup_quad_cksum(void*, size_t, des_cblock*, void*, void*, int);
 
 /*
  * krb_mk_safe() constructs an AUTH_MSG_SAFE message.  It takes some
@@ -105,9 +108,23 @@ krb_mk_safe(void *in, void *out, u_int32_t length, des_cblock *key,
 
     p += krb_put_int(lsb_time(tv.tv_sec, sender, receiver), p, 4);
 
-    des_quad_cksum((des_cblock*)start, (des_cblock*)p, p - start, 2, key);
-    if(HOST_BYTE_ORDER)
-	swap_u_16(p);
+    {
+	/* We are faking big endian mode, so we need to fix the
+	 * checksum (that is byte order dependent). We always send a
+	 * checksum of the new type, unless we know that we are
+	 * talking to an old client (this requires a call to
+	 * krb_rd_safe first).  
+	 */
+	unsigned char new_checksum[16];
+	unsigned char old_checksum[16];
+	fixup_quad_cksum(start, p - start, key, new_checksum, old_checksum, 0);
+	
+	if((dqc_type == DES_QUAD_GUESS && DES_QUAD_DEFAULT == DES_QUAD_OLD) || 
+	   dqc_type == DES_QUAD_OLD)
+	    memcpy(p, old_checksum, 16);
+	else
+	    memcpy(p, new_checksum, 16);
+    }
     p += 16;
 
     return p - (unsigned char*)out;
