@@ -38,7 +38,7 @@
 
 #include "krb_locl.h"
 
-RCSID("$Id: realm_parse.c,v 1.9 1997/04/30 04:32:46 assar Exp $");
+RCSID("$Id: realm_parse.c,v 1.13 1997/12/05 09:05:25 joda Exp $");
 
 static int
 realm_parse(char *realm, int length, const char *file)
@@ -51,10 +51,12 @@ realm_parse(char *realm, int length, const char *file)
 	return -1;
     
     while(fgets(tr, sizeof(tr), F)){
-	p = strtok(tr, " \t\n\r");
-	if(strcasecmp(p, realm) == 0){
+	char *unused = NULL;
+	p = strtok_r(tr, " \t\n\r", &unused);
+	if(p && strcasecmp(p, realm) == 0){
 	    fclose(F);
 	    strncpy(realm, p, length);
+	    realm[length - 1] = '\0';
 	    return 0;
 	}
     }
@@ -62,26 +64,13 @@ realm_parse(char *realm, int length, const char *file)
     return -1;
 }
 
-static const char *const files[] = KRB_CNF_FILES;
-
 int
 krb_realm_parse(char *realm, int length)
 {
     int i;
-  
-    const char *dir = getenv("KRBCONFDIR");
-
-    /* First try user specified file */
-    if (dir != 0) {
-      char fname[MaxPathLen];
-
-      if(k_concat(fname, sizeof(fname), dir, "/krb.conf", NULL) == 0)
-	  if (realm_parse(realm, length, fname) == 0)
-	      return 0;
-    }
-
-    for (i = 0; files[i] != NULL; i++)
-	if (realm_parse(realm, length, files[i]) == 0)
+    char file[128];
+    for(i = 0; krb_get_krbconf(i, file, sizeof(file)) == 0; i++)
+	if (realm_parse(realm, length, file) == 0)
 	    return 0;
     return -1;
 }
