@@ -20,8 +20,9 @@ or implied warranty.
   */
 
 #include "krb_locl.h"
+#include <base64.h>
 
-RCSID("$Id: send_to_kdc.c,v 1.47 1997/11/07 17:31:38 bg Exp $");
+RCSID("$Id: send_to_kdc.c,v 1.49 1997/12/05 01:57:09 assar Exp $");
 
 struct host {
     struct sockaddr_in addr;
@@ -250,12 +251,14 @@ static int http_connect(int s, struct sockaddr_in *adr)
     return connect(s, (struct sockaddr*)&sin, sizeof(sin));
 }
 
-static int http_send(int s, struct sockaddr_in* adr, KTEXT pkt)
+static int
+http_send(int s, struct sockaddr_in* adr, KTEXT pkt)
 {
     char *str;
     char *msg;
 
-    base64_encode(pkt->dat, pkt->length, &str);
+    if(base64_encode(pkt->dat, pkt->length, &str) < 0)
+	return -1;
     if(getenv(PROXY_VAR)){
 	asprintf(&msg, "GET http://%s:%d/%s HTTP/1.0\r\n\r\n",
 		 inet_ntoa(adr->sin_addr),
@@ -264,6 +267,9 @@ static int http_send(int s, struct sockaddr_in* adr, KTEXT pkt)
     }else
 	asprintf(&msg, "GET %s HTTP/1.0\r\n\r\n", str);
     free(str);
+
+    if (msg == NULL)
+	return -1;
 	
     if(send(s, msg, strlen(msg), 0) != strlen(msg)){
 	free(msg);
@@ -273,10 +279,14 @@ static int http_send(int s, struct sockaddr_in* adr, KTEXT pkt)
     return 0;
 }
 
-static int http_recv(void *buf, size_t len, KTEXT rpkt)
+static int
+http_recv(void *buf, size_t len, KTEXT rpkt)
 {
     char *p;
     char *tmp = malloc(len + 1);
+
+    if (tmp == NULL)
+	return -1;
     memcpy(tmp, buf, len);
     tmp[len] = 0;
     p = strstr(tmp, "\r\n\r\n");
@@ -358,9 +368,9 @@ send_recv(KTEXT pkt, KTEXT rpkt, int proto, struct sockaddr_in *adr,
     return TRUE;
 }
 
-#if defined(linux)
-/* Linux libc is broken probably somewhere in nsw_hosts.o,
- * for now keep this kludge */
+#if defined(linux) && defined(HAVE__DNS_GETHOSTBYNAME)
+/* Linux libc 5.3 is broken probably somewhere in nsw_hosts.o,
+ * for now keep this kludge. */
 static
 struct hostent *gethostbyname(const char *name)
 {
