@@ -38,7 +38,7 @@
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
-RCSID("$Id: sl.c,v 1.11 1997/04/11 14:30:22 assar Exp $");
+RCSID("$Id: sl.c,v 1.15 1997/10/19 23:12:40 assar Exp $");
 #endif
 
 #include "sl_locl.h"
@@ -89,10 +89,13 @@ sl_help (SL_cmd *cmds, int argc, char **argv)
     } else { 
 	c = sl_match (cmds, argv[1], 0);
 	if (c == NULL)
-	    printf ("No such command: %s. Try \"help\" for a list of all commands\n",
+	    printf ("No such command: %s. "
+		    "Try \"help\" for a list of all commands\n",
 		    argv[1]);
 	else {
 	    printf ("%s\t%s", c->name, c->usage);
+	    if(c->help && *c->help)
+		printf ("%s\n", c->help);
 	    if((++c)->name && c->func == NULL) {
 		printf ("\nSynonyms:");
 		while (c->name && c->func == NULL)
@@ -130,11 +133,22 @@ add_history(char *p)
 
 #endif
 
+int
+sl_command(SL_cmd *cmds, int argc, char **argv)
+{
+    SL_cmd *c;
+    c = sl_match (cmds, argv[0], 0);
+    if (c == NULL)
+	return -1;
+    return (*c->func)(argc, argv);
+}
+
 int
 sl_loop (SL_cmd *cmds, char *prompt)
 {
     unsigned max_count;
     char **ptr;
+    int ret;
 
     max_count = 17;
     ptr = malloc(max_count * sizeof(*ptr));
@@ -149,6 +163,7 @@ sl_loop (SL_cmd *cmds, char *prompt)
 	unsigned count;
 	SL_cmd *c;
 
+	ret = 0;
 	buf = readline(prompt);
 	if(buf == NULL)
 	    break;
@@ -157,7 +172,7 @@ sl_loop (SL_cmd *cmds, char *prompt)
 	    add_history(buf);
 	count = 0;
 	{
-	    char *foo;
+	    char *foo = NULL;
 	    char *p;
 
 	    for(p = strtok_r (buf, " \t", &foo);
@@ -178,9 +193,13 @@ sl_loop (SL_cmd *cmds, char *prompt)
 	}
 	if (count > 0) {
 	    c = sl_match (cmds, ptr[0], 0);
-	    if (c)
-		(*c->func)(count, ptr);
-	    else
+	    if (c) {
+		ret = (*c->func)(count, ptr);
+		if (ret != 0) {
+		    free (buf);
+		    break;
+		}
+	    } else
 		printf ("Unrecognized command: %s\n", ptr[0]);
 	}
 	free(buf);
