@@ -9,11 +9,7 @@
 #include "config.h"
 #include "protos.h"
 
-RCSID("$Id: kerberos.c,v 1.50 1996/11/24 05:19:35 joda Exp $");
-
-#ifdef SOCKS
-#include <socks.h>
-#endif
+RCSID("$Id: kerberos.c,v 1.75 1997/12/05 04:42:05 assar Exp $");
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -27,51 +23,55 @@ RCSID("$Id: kerberos.c,v 1.50 1996/11/24 05:19:35 joda Exp $");
 #ifdef TIME_WITH_SYS_TIME
 #include <sys/time.h>
 #include <time.h>
-#else
-#ifdef HAVE_SYS_TIME_H
+#elif defined(HAVE_SYS_TIME_H)
 #include <sys/time.h>
 #else
 #include <time.h>
-#endif /* !HAVE_SYS_TIME_H */
-#endif /* !TIME_WITH_SYS_TIME */
+#endif
 
 #ifdef HAVE_SYS_SELECT_H
 #include <sys/select.h>
 #endif
 
+#include <errno.h>
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
+#ifdef HAVE_SYS_SOCKET_H
+#include <sys/socket.h>
+#endif
+#ifdef HAVE_NETINET_IN_H
+#include <netinet/in.h>
+#endif
+#ifdef HAVE_ARPA_INET_H
+#include <arpa/inet.h>
+#endif
+
 #ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
 #endif
 #ifdef HAVE_FCNTL_H
 #include <fcntl.h>
 #endif
-#ifdef HAVE_SYS_IOCTL_H
+#if defined(HAVE_SYS_IOCTL_H) && SunOS != 4
 #include <sys/ioctl.h>
 #endif
 #ifdef HAVE_SYS_FILIO_H
 #include <sys/filio.h>
 #endif /* HAVE_SYS_FILIO_H */
 
-#include <errno.h>
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-
-#ifdef HAVE_NETINET_IN_H
-#include <netinet/in.h>
-#endif
-#ifdef HAVE_ARPA_INET_H
-#include <arpa/inet.h>
-#endif
-#ifdef HAVE_SYS_SOCKET_H
-#include <sys/socket.h>
-#endif
-
 #ifdef HAVE_NETDB_H
 #include <netdb.h>
 #endif
+#include <err.h>
+
+#ifdef SOCKS
+#include <socks.h>
+#endif
 
 #include <roken.h>
+#include <base64.h>
 
 #include <des.h>
 #include <krb.h>
@@ -79,9 +79,9 @@ RCSID("$Id: kerberos.c,v 1.50 1996/11/24 05:19:35 joda Exp $");
 #include <prot.h>
 #include <klog.h>
 
-#include <kdc.h>
+#include <krb_log.h>
 
-static char *progname;
+#include <kdc.h>
 
 static des_key_schedule master_key_schedule;
 static des_cblock master_key;
@@ -93,8 +93,7 @@ static char *lt;
 static int more;
 
 static int mflag;		/* Are we invoked manually? */
-static int lflag;		/* Have we set an alterate log file? */
-static char *log_file;		/* name of alt. log file */
+static char *log_file = KRBLOG;	/* name of alt. log file */
 static int nflag;		/* don't check max age */
 static int rflag;		/* alternate realm specified */
 
@@ -117,8 +116,9 @@ static void
 usage(void)
 {
     fprintf(stderr, "Usage: %s [-s] [-m] [-n] [-p pause_seconds]"
-	    " [-a max_age] [-l log_file] [-r realm] [database_pathname]\n",
-	    progname);
+	    " [-a max_age] [-l log_file] [-i address_to_listen_on]"
+	    " [-r realm] [database_pathname]\n",
+	    __progname);
     exit(1);
 }
 
@@ -139,7 +139,7 @@ kerb_err_reply(int f, struct sockaddr_in *client, int err, char *string)
     cr_err_reply(e_pkt, req_name_ptr, req_inst_ptr, req_realm_ptr,
 		 req_time_ws, err, e_msg);
     sendto(f, (char*)e_pkt->dat, e_pkt->length, 0, (struct sockaddr *)client,
-	   S_AD_SZ);
+	   sizeof(*client));
 }
 
 static void
@@ -151,7 +151,9 @@ hang(void)
 	    pause();
     } else {
 	char buf[256];
-	sprintf(buf,  "Kerberos will wait %d seconds before dying so as not to loop init", pause_int);
+	snprintf(buf, sizeof(buf),
+		 "Kerberos will wait %d seconds before dying so as not to loop init",
+		 pause_int);
 	klog(L_KRB_PERR, buf);
 	sleep(pause_int);
 	klog(L_KRB_PERR, "Do svedania....\n");
@@ -250,7 +252,10 @@ set_tgtkey(char *r)
 
 
 static int
-kerberos(unsigned char *buf, int len, struct in_addr client, KTEXT rpkt)
+kerberos(unsigned char *buf, int len,
+	 char *proto, struct sockaddr_in *client,
+	 struct sockaddr_in *server,
+	 KTEXT rpkt)
 {
     int pvno;
     int msg_type;
@@ -294,8 +299,11 @@ kerberos(unsigned char *buf, int len, struct in_addr client, KTEXT rpkt)
 	p += krb_get_int(p, &req_time, 4, lsb);
 	life = *p++;
 	p += krb_get_nir(p, service, sinst, NULL);
-	klog(L_INI_REQ, "AS REQ %s.%s@%s for %s.%s from %s", 
-	     name, inst, realm, service, sinst, inet_ntoa(client));
+	klog(L_INI_REQ,
+	     "AS REQ %s.%s@%s for %s.%s from %s (%s/%u)", 
+	     name, inst, realm, service, sinst,
+	     inet_ntoa(client->sin_addr),
+	     proto, ntohs(server->sin_port));
 	if((err = check_princ(name, inst, 0, &a_name))){
 	    strcpy((char*)rpkt->dat, krb_get_err_text(err));
 	    return err;
@@ -312,7 +320,8 @@ kerberos(unsigned char *buf, int len, struct in_addr client, KTEXT rpkt)
 	copy_to_key(&s_name.key_low, &s_name.key_high, key);
 	unseal(&key);
 	krb_create_ticket(tk, flags, a_name.name, a_name.instance, 
-			  local_realm, client.s_addr, session, 
+			  local_realm, client->sin_addr.s_addr,
+			  session, 
 			  life, kerb_time.tv_sec, 
 			  s_name.name, s_name.instance, &key);
 	copy_to_key(&a_name.key_low, &a_name.key_high, key);
@@ -332,8 +341,10 @@ kerberos(unsigned char *buf, int len, struct in_addr client, KTEXT rpkt)
     case AUTH_MSG_APPL_REQUEST:
 	strcpy(realm, (char*)buf + 3);
 	if((err = set_tgtkey(realm))){
-	    msg = klog(L_ERR_UNK, "Unknown realm %s from %s", 
-		       realm, inet_ntoa(client));
+	    msg = klog(L_ERR_UNK,
+		       "Unknown realm %s from %s (%s/%u)", 
+		       realm, inet_ntoa(client->sin_addr),
+		       proto, ntohs(server->sin_port));
 	    strcpy((char*)rpkt->dat, msg);
 	    return err;
 	}
@@ -342,22 +353,32 @@ kerberos(unsigned char *buf, int len, struct in_addr client, KTEXT rpkt)
 	auth->length = p - buf;
 	memcpy(auth->dat, buf, auth->length);
 	err = krb_rd_req(auth, KRB_TICKET_GRANTING_TICKET,
-			 realm, client.s_addr, &ad, 0);
+			 realm, client->sin_addr.s_addr, &ad, 0);
 	if(err){
-	    msg = klog(L_ERR_UNK, "krb_rd_req from %s: %s", 
-		       inet_ntoa(client), krb_get_err_text(err));
+	    msg = klog(L_ERR_UNK,
+		       "krb_rd_req from %s (%s/%u): %s", 
+		       inet_ntoa(client->sin_addr),
+		       proto,
+		       ntohs(server->sin_port),
+		       krb_get_err_text(err));
 	    strcpy((char*)rpkt->dat, msg);
 	    return err;
 	}
 	p += krb_get_int(p, &req_time, 4, lsb);
 	life = *p++;
 	p += krb_get_nir(p, service, sinst, NULL);
-	klog(L_APPL_REQ, "APPL REQ %s.%s@%s for %s.%s from %s",
-	     ad.pname, ad.pinst, ad.prealm, 
-	     service, sinst, 
-	     inet_ntoa(client));
+	klog(L_APPL_REQ,
+	     "APPL REQ %s.%s@%s for %s.%s from %s (%s/%u)",
+	     ad.pname, ad.pinst, ad.prealm,
+	     service, sinst,
+	     inet_ntoa(client->sin_addr),
+	     proto,
+	     ntohs(server->sin_port));
+
 	if(strcmp(ad.prealm, realm)){
-	    strcpy((char*)rpkt->dat, "Can't hop realms");
+	    msg = klog(L_ERR_UNK, "Can't hop realms: %s -> %s", 
+		       realm, ad.prealm);
+	    strcpy((char*)rpkt->dat, msg);
 	    return KERB_ERR_PRINCIPAL_UNKNOWN;
 	}
 
@@ -381,7 +402,8 @@ kerberos(unsigned char *buf, int len, struct in_addr client, KTEXT rpkt)
 	unseal(&key);
 	des_new_random_key(&session);
 	krb_create_ticket(tk, flags, ad.pname, ad.pinst, ad.prealm,
-			  client.s_addr, &session, life, kerb_time.tv_sec,
+			  client->sin_addr.s_addr, &session,
+			  life, kerb_time.tv_sec,
 			  s_name.name, s_name.instance,
 			  &key);
 	
@@ -405,8 +427,12 @@ kerberos(unsigned char *buf, int len, struct in_addr client, KTEXT rpkt)
     case AUTH_MSG_ERR_REPLY:
 	return -1;
     default:
-	msg = klog(L_KRB_PERR, "Unknown message type: %d from %s", 
-		   msg_type, inet_ntoa(client));
+	msg = klog(L_KRB_PERR,
+		   "Unknown message type: %d from %s (%s/%u)", 
+		   msg_type,
+		   inet_ntoa(client->sin_addr),
+		   proto,
+		   ntohs(server->sin_port));
 	strcpy((char*)rpkt->dat, msg);
 	return KFAILURE;
     }
@@ -414,12 +440,23 @@ kerberos(unsigned char *buf, int len, struct in_addr client, KTEXT rpkt)
 
 
 static void
-kerberos_wrap(int s, KTEXT data, struct sockaddr_in *client)
+kerberos_wrap(int s, KTEXT data, char *proto, struct sockaddr_in *client, 
+	      struct sockaddr_in *server)
 {
     KTEXT_ST pkt;
-    int err = kerberos(data->dat, data->length, client->sin_addr, &pkt);
+    int http_flag = strcmp(proto, "http") == 0;
+    int err = kerberos(data->dat, data->length, proto, client, server, &pkt);
     if(err == -1)
 	return;
+    if(http_flag){
+	const char *msg = 
+	    "HTTP/1.1 200 OK\r\n"
+	    "Server: KTH-KRB/" VERSION "\r\n"
+	    "Content-type: application/octet-stream\r\n"
+	    "Content-transfer-encoding: binary\r\n\r\n";
+	sendto(s, msg, strlen(msg), 0, (struct sockaddr *)client,
+	       sizeof(*client));
+    }
     if(err){
 	kerb_err_reply(s, client, err, (char*)pkt.dat);
 	return;
@@ -487,34 +524,32 @@ struct descr{
     KTEXT_ST buf;
     int type;
     int timeout;
+    struct sockaddr_in addr;
 };
 
 static void
 mksocket(struct descr *d, struct in_addr addr, int type, 
 	 const char *service, int port)
 {
-    struct sockaddr_in sina;
     int     on = 1;
     int sock;
 
     memset(d, 0, sizeof(struct descr));
-    if ((sock = socket(AF_INET, type, 0)) < 0) {
-	fprintf(stderr, "%s: Can't open socket\n", progname);
-	exit(1);
-    }
+    if ((sock = socket(AF_INET, type, 0)) < 0)
+	err (1, "socket");
+#if defined(SO_REUSEADDR) && defined(HAVE_SETSOCKOPT)
     if (setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, (void *)&on,
 		   sizeof(on)) < 0)
-	fprintf(stderr, "%s: setsockopt (SO_REUSEADDR)\n", progname);
-    memset(&sina, 0, sizeof(sina));
-    sina.sin_family = AF_INET;
-    sina.sin_port   = port;
-    sina.sin_addr   = addr;
-    if (bind(sock, (struct sockaddr *)&sina, sizeof(sina)) < 0) {
-	fprintf(stderr, "%s: Can't bind socket '%s/%s' (%d)\n", progname, 
-		service, (type == SOCK_DGRAM) ? "udp" : "tcp", 
-		ntohs(sina.sin_port));
-	exit(1);
-    }
+	warn ("setsockopt (SO_REUSEADDR)");
+#endif
+    memset(&d->addr, 0, sizeof(d->addr));
+    d->addr.sin_family = AF_INET;
+    d->addr.sin_port   = port;
+    d->addr.sin_addr   = addr;
+    if (bind(sock, (struct sockaddr *)&d->addr, sizeof(d->addr)) < 0)
+	err (1, "bind '%s/%s' (%d)",
+	     service, (type == SOCK_DGRAM) ? "udp" : "tcp",
+	     ntohs(d->addr.sin_port));
     
     if(type == SOCK_STREAM)
 	listen(sock, SOMAXCONN);
@@ -525,6 +560,113 @@ mksocket(struct descr *d, struct in_addr addr, int type,
 
 static void loop(struct descr *fds, int maxfd);
 
+struct port_spec {
+    int port;
+    int type;
+};
+
+static int
+add_port(struct port_spec **ports, int *num_ports, int port, int type)
+{
+    struct port_spec *tmp;
+    tmp = realloc(*ports, (*num_ports + 1) * sizeof(*tmp));
+    if(tmp == NULL)
+	return ENOMEM;
+    *ports = tmp;
+    tmp[*num_ports].port = port;
+    tmp[*num_ports].type = type;
+    (*num_ports)++;
+    return 0;
+}
+
+void make_sockets(char *port_spec, struct in_addr *i_addr, 
+		  struct descr **fds, int *nfds)
+{
+    int tp;
+    struct in_addr *a;
+    char *p, *q, *pos = NULL;
+    struct servent *sp;
+    struct port_spec *ports = NULL;
+    int num_ports = 0;
+    int i, j;
+    
+
+    for(p = strtok_r(port_spec, ", \t", &pos); 
+	p; 
+	p = strtok_r(NULL, ", \t", &pos)){
+	if(strcmp(p, "+") == 0){
+	    add_port(&ports, &num_ports, 88, SOCK_DGRAM);
+	    add_port(&ports, &num_ports, 88, SOCK_STREAM);
+	    add_port(&ports, &num_ports, 750, SOCK_DGRAM);
+	    add_port(&ports, &num_ports, 750, SOCK_STREAM);
+	}else{
+	    q = strchr(p, '/');
+	    if(q){
+		*q = 0;
+		q++;
+	    }
+	    sp = getservbyname(p, q);
+	    if(sp)
+		tp = ntohs(sp->s_port);
+	    else if(sscanf(p, "%d", &tp) != 1) {
+		warnx("Unknown port: %s%s%s", p, q ? "/" : "", q ? q : "");
+		continue;
+	    }
+	    if(q){
+		if(strcasecmp(q, "tcp") == 0)
+		    add_port(&ports, &num_ports, tp, SOCK_STREAM);
+		else if(strcasecmp(q, "udp") == 0)
+		    add_port(&ports, &num_ports, tp, SOCK_DGRAM);
+		else
+		    warnx("Unknown protocol type: %s", q);
+	    }else{
+		add_port(&ports, &num_ports, tp, SOCK_DGRAM);
+		add_port(&ports, &num_ports, tp, SOCK_STREAM);
+	    }
+	}
+    }
+
+    if(num_ports == 0)
+	errx(1, "No valid ports specified!");
+    
+    if (i_addr) {
+	*nfds = 1;
+	a = malloc(sizeof(*a) * *nfds);
+	if (a == NULL)
+	    errx (1, "Failed to allocate %u bytes",
+		  sizeof(*a) * *nfds);
+	memcpy(a, i_addr, sizeof(struct in_addr));
+    } else
+	*nfds = k_get_all_addrs (&a);
+    if (*nfds < 0) {
+	struct in_addr any;
+
+	any.s_addr = INADDR_ANY;
+
+	warnx ("Could not get local addresses, binding to INADDR_ANY");
+	*nfds = 1;
+	a = malloc(sizeof(*a) * *nfds);
+	if (a == NULL)
+	    errx (1, "Failed to allocate %u bytes",
+		  sizeof(*a) * *nfds);
+	memcpy(a, &any, sizeof(struct in_addr));
+    }
+    *fds = malloc(*nfds * num_ports * sizeof(**fds));
+    if (*fds == NULL)
+	errx (1, "Failed to allocate %u bytes",
+	      *nfds * num_ports * sizeof(**fds));
+    for (i = 0; i < *nfds; i++) {
+	for(j = 0; j < num_ports; j++) {
+	    mksocket(*fds + num_ports * i + j, a[i], 
+		     ports[j].type, "", htons(ports[j].port));
+	}
+    }
+    *nfds *= num_ports;
+    free(ports);
+    free (a);
+}
+
+
 int
 main(int argc, char **argv)
 {
@@ -532,15 +674,17 @@ main(int argc, char **argv)
     int c;
     struct descr *fds;
     int nfds;
-    int i;
     int n;
     int     kerror;
+    int i_flag = 0;
+    struct in_addr i_addr;
+    char *port_spec = "+";
 
     umask(077);		/* Create protected files */
 
-    progname = argv[0];
+    set_progname (argv[0]);
 
-    while ((c = getopt(argc, argv, "snmp:a:l:r:")) != EOF) {
+    while ((c = getopt(argc, argv, "snmp:P:a:l:r:i:")) != EOF) {
 	switch(c) {
 	case 's':
 	    /*
@@ -550,13 +694,6 @@ main(int argc, char **argv)
 		max_age = ONE_DAY;	/* 24 hours */
 	    if (pause_int == -1)
 		pause_int = FIVE_MINUTES; /* 5 minutes */
-	    if (lflag == 0) {
-#if 0
-		/* this is only silly */
-		log_file = KRBSLAVELOG;
-#endif
-		lflag++;
-	    }
 	    break;
 	case 'n':
 	    max_age = -1;	/* don't check max age. */
@@ -575,6 +712,9 @@ main(int argc, char **argv)
 		usage();
 	    }
 	    break;
+	case 'P':
+	    port_spec = optarg;
+	    break;
 	case 'a':
 	    /* Set max age. */
 	    if (!isdigit(optarg[0])) 
@@ -587,7 +727,6 @@ main(int argc, char **argv)
 	    break;
 	case 'l':
 	    /* Set alternate log file */
-	    lflag++;
 	    log_file = optarg;
 	    break;
 	case 'r':
@@ -595,12 +734,20 @@ main(int argc, char **argv)
 	    rflag++;
 	    strcpy(local_realm, optarg);
 	    break;
+	case 'i':
+	    /* Only listen on this address */
+	    if(inet_aton (optarg, &i_addr) == 0) {
+		fprintf (stderr, "Bad address: %s\n", optarg);
+		exit (1);
+	    }
+	    ++i_flag;
+	    break;
 	default:
 	    usage();
 	    break;
 	}
     }
-
+    
     if (optind == (argc-1)) {
 	if (kerb_db_set_name(argv[optind]) != 0) {
 	    fprintf(stderr, "Could not set alternate database name\n");
@@ -623,58 +770,16 @@ main(int argc, char **argv)
     if (mflag)
 	printf("\tMaster key will be entered manually\n");
     
-    printf("\tLog file is %s\n", lflag ? log_file : KRBLOG);
+    printf("\tLog file is %s\n", log_file);
 
-    if (lflag)
-	kset_logfile(log_file);
+    kset_logfile(log_file);
     
     /* find our hostname, and use it as the instance */
-    if (k_gethostname(k_instance, INST_SZ)) {
-	fprintf(stderr, "%s: gethostname error\n", progname);
-	exit(1);
-    }
+    if (gethostname(k_instance, INST_SZ))
+	err (1, "gethostname");
 
-    /*
-     * Yes this looks backwards but it has to be this way to enable a
-     * smooth migration to the new port 88.
-     */
-    {
-      int p1, p2;
-      struct in_addr *a;
+    make_sockets(port_spec, i_flag ? &i_addr : NULL, &fds, &nfds);
 
-      p1 = k_getportbyname ("kerberos-iv", "udp", htons(750));
-      p2 = k_getportbyname ("kerberos-sec", "udp", htons(88));
-
-      if (p1 == p2)
-	{
-	  fprintf(stderr, "Either define kerberos-iv/udp as 750\n");
-	  fprintf(stderr, "      and kerberos-sec/udp as 88\n");
-	  fprintf(stderr, "or the other way around!");
-	  exit(1);
-	}
-
-      nfds = k_get_all_addrs (&a);
-      if (nfds < 0) {
-	   struct in_addr any;
-
-	   any.s_addr = INADDR_ANY;
-
-	   fprintf (stderr, "Could not get local addresses, "
-		    "binding to INADDR_ANY\n");
-	   nfds = 1;
-	   a = malloc(sizeof(*a) * nfds);
-	   memcpy(a, &any, sizeof(struct in_addr));
-      }
-      nfds *= 4;
-      fds = (struct descr*)malloc(nfds * sizeof(struct descr));
-      for (i = 0; i < nfds/4; i++) {
-	  mksocket(fds + 4 * i + 0, a[i], SOCK_DGRAM, "kerberos-iv", p1);
-	  mksocket(fds + 4 * i + 1, a[i], SOCK_DGRAM, "kerberos-sec", p2);
-	  mksocket(fds + 4 * i + 2, a[i], SOCK_STREAM, "kerberos-iv", p1);
-	  mksocket(fds + 4 * i + 3, a[i], SOCK_STREAM, "kerberos-sec", p2);
-      }
-      free (a);
-    }
     /* do all the database and cache inits */
     if ((n = kerb_init())) {
 	if (mflag) {
@@ -693,7 +798,7 @@ main(int argc, char **argv)
     
     /* setup master key */
     if (kdb_get_master_key (mflag, &master_key, master_key_schedule) != 0) {
-      klog (L_KRB_PERR, "kerberos: couldn't get master key.\n");
+      klog (L_KRB_PERR, "kerberos: couldn't get master key.");
       exit (1);
     }
     kerror = kdb_verify_master_key (&master_key, master_key_schedule, stdout);
@@ -740,6 +845,98 @@ main(int argc, char **argv)
 }
 
 
+void
+read_socket(struct descr *n)
+{
+    int b;
+    struct sockaddr_in from;
+    int fromlen = sizeof(from);
+    b = recvfrom(n->s, n->buf.dat + n->buf.length, 
+		 MAX_PKT_LEN - n->buf.length, 0, 
+		 (struct sockaddr *)&from, &fromlen);
+    if(b < 0){
+	if(n->type == SOCK_STREAM){
+	    close(n->s);
+	    n->s = -1;
+	}
+	n->buf.length = 0;
+	return;
+    }
+    n->buf.length += b;
+    if(n->type == SOCK_STREAM){
+	char *proto = "tcp";
+	if(n->buf.length > 4 && 
+	   strncmp((char *)n->buf.dat, "GET ", 4) == 0 &&
+	   strncmp((char *)n->buf.dat + n->buf.length - 4, 
+		   "\r\n\r\n", 4) == 0){
+	    char *p;
+	    char *save = NULL;
+
+	    n->buf.dat[n->buf.length - 1] = 0;
+	    strtok_r(n->buf.dat, " \t\r\n", &save);
+	    p = strtok_r(NULL, " \t\r\n", &save);
+	    if(p == NULL)
+		p = "";
+	    if(*p == '/') p++;
+	    n->buf.length = base64_decode(p, n->buf.dat);
+	    if(n->buf.length <= 0){
+		const char *msg = 
+		    "HTTP/1.1 404 Not found\r\n"
+		    "Server: KTH-KRB/" VERSION "\r\n"
+		    "Content-type: text/html\r\n"
+		    "Content-transfer-encoding: 8bit\r\n\r\n"
+		    "<TITLE>404 Not found</TITLE>\r\n"
+		    "<H1>404 Not found</H1>\r\n"
+		    "That page does not exist. Information about "
+		    "<A HREF=\"http://www.pdc.kth.se/kth-krb\">KTH-KRB</A> "
+		    "is available elsewhere.\r\n";
+		fromlen = sizeof(from);
+		if(getpeername(n->s,(struct sockaddr*)&from, &fromlen) == 0)
+		    klog(L_KRB_PERR, "Unknown HTTP request from %s", 
+			 inet_ntoa(from.sin_addr));
+		else
+		    klog(L_KRB_PERR, "Unknown HTTP request from <unknown>");
+		write(n->s, msg, strlen(msg));
+		close(n->s);
+		n->s = -1;
+		n->buf.length = 0;
+		return;
+	    }
+	    proto = "http";
+	    b = 0;
+	}
+	else if(n->buf.length >= 4 && n->buf.dat[0] == 0){
+	    /* if this is a new type of packet (with
+	       the length attached to the head of the
+	       packet), and there is no more data to
+	       be read, fake an old packet, so the
+	       code below will work */
+	    u_int32_t len;
+	    krb_get_int(n->buf.dat, &len, 4, 0);
+	    if(n->buf.length == len + 4){
+		memmove(n->buf.dat, n->buf.dat + 4, len);
+		b = 0;
+	    }
+	}
+	if(b == 0){
+	    /* handle request if there are 
+	       no more bytes to read */
+	    fromlen = sizeof(from);
+	    getpeername(n->s,(struct sockaddr*)&from, &fromlen);
+	    kerberos_wrap(n->s, &n->buf, proto, &from,
+			  &n->addr);
+	    n->buf.length = 0;
+	    close(n->s);
+	    n->s = -1;
+	}
+    }else{
+	/* udp packets are atomic */
+	kerberos_wrap(n->s, &n->buf, "udp", &from,
+		      &n->addr);
+	n->buf.length = 0;
+    }
+}
+
 static void
 loop(struct descr *fds, int nfds)
 {
@@ -749,6 +946,7 @@ loop(struct descr *fds, int nfds)
 	struct timeval tv;
 	int maxfd = 0;
 	struct descr *n, *minfree;
+	int accepted; /* accept at most one socket per `round' */
 	
 	FD_ZERO(&readfds);
 	gettimeofday(&tv, NULL);
@@ -782,13 +980,17 @@ loop(struct descr *fds, int nfds)
 	    }
 	}
 	ret = select(maxfd + 1, &readfds, 0, 0, 0);
+	accepted = 0;
 	for (n = fds; n < fds + nfds; n++){
 	    if(n->s < 0) continue;
 	    if (FD_ISSET(n->s, &readfds)){
 		if(n->type == SOCK_STREAM && n->timeout == 0){
 		    /* add accepted socket to list of sockets we are
                        selecting on */
-		    int s = accept(n->s, NULL, 0);
+		    int s;
+		    if(accepted) continue;
+		    accepted = 1;
+		    s = accept(n->s, NULL, 0);
 		    if(minfree == NULL){
 			kerb_err_reply(s, NULL, KFAILURE, "Out of memory");
 			close(s);
@@ -797,39 +999,11 @@ loop(struct descr *fds, int nfds)
 			minfree->type = SOCK_STREAM;
 			gettimeofday(&tv, NULL);
 			minfree->timeout = tv.tv_sec + 4; /* XXX */
+			minfree->buf.length = 0;
+			memcpy(&minfree->addr, &n->addr, sizeof(minfree->addr));
 		    }
-		}else{
-		    int b;
-		    struct sockaddr_in from;
-		    int fromlen = sizeof(from);
-		    b = recvfrom(n->s, n->buf.dat + n->buf.length, 
-				 MAX_PKT_LEN - n->buf.length, 0, 
-				 (struct sockaddr *)&from, &fromlen);
-		    if(b < 0){
-			if(n->type == SOCK_STREAM){
-			    close(n->s);
-			    n->s = -1;
-			}
-			n->buf.length = 0;
-			continue;
-		    }
-		    n->buf.length += b;
-		    if(n->type == SOCK_STREAM && b == 0){
-			/* handle request if there are 
-			   no more bytes to read */
-			fromlen = sizeof(from);
-			getpeername(n->s, (struct sockaddr*)&from, &fromlen);
-			kerberos_wrap(n->s, &n->buf, &from);
-			n->buf.length = 0;
-			close(n->s);
-			n->s = -1;
-		    }
-		    if(n->type == SOCK_DGRAM){
-			/* udp packets are atomic */
-			kerberos_wrap(n->s, &n->buf, &from);
-			n->buf.length = 0;
-		    }
-		}
+		}else
+		    read_socket(n);
 	    }
 	}
     }
