@@ -19,7 +19,7 @@ provided "as is" without express or implied warranty.
 
 #include "slav_locl.h"
 
-RCSID("$Id: kprop.c,v 1.22 1996/11/19 22:48:50 assar Exp $");
+RCSID("$Id: kprop.c,v 1.30 1997/12/05 01:34:49 assar Exp $");
 
 #include "kprop.h"
 
@@ -43,14 +43,6 @@ struct slave_host {
     struct slave_host *next;
 };
 
-static void
-Death(char *s)
-{
-    fprintf(stderr, "kprop: ");
-    perror(s);
-    exit(1);
-}
-
 static
 int get_slaves(struct slave_host **psl, char *file, time_t ok_mtime)
 {
@@ -63,10 +55,8 @@ int get_slaves(struct slave_host **psl, char *file, time_t ok_mtime)
     char   *ppath;
     struct stat stbuf;
 
-    if ((fin = fopen(file, "r")) == NULL) {
-	fprintf(stderr, "Can't open slave host file, '%s'.\n", file);
-	exit (1);
-    }
+    if ((fin = fopen(file, "r")) == NULL)
+	err (1, "open(%s)", file);
     strcpy(path, file);
     if ((ppath = strrchr(path, '/'))) {
 	ppath += 1;
@@ -79,8 +69,8 @@ int get_slaves(struct slave_host **psl, char *file, time_t ok_mtime)
 	    *pc = '\0';
 	} else {
 	    if(strlen(namebuf) == sizeof(namebuf) - 1){
-		fprintf(stderr, "Hostname too long (>= %d chars) in '%s'.\n",
-			sizeof(namebuf), file);
+		warnx ("Hostname too long (>= %d chars) in '%s'.",
+		       (int) sizeof(namebuf), file);
 		do{
 		    if(fgets(namebuf, sizeof(namebuf), fin) == NULL)
 			break;
@@ -92,33 +82,31 @@ int get_slaves(struct slave_host **psl, char *file, time_t ok_mtime)
 	    continue;
 	host = gethostbyname(namebuf);
 	if (host == NULL) {
-	    fprintf(stderr, "Ignoring unknown host '%s' in '%s'.\n", 
-		    namebuf, file);
+	    warnx ("Ignoring host '%s' in '%s': %s", 
+		   namebuf, file,
+#ifdef HAVE_H_ERRNO
+		   hstrerror(h_errno)
+#else
+		   "unknown error"
+#endif
+		   );
 	    continue;
 	}
 	(*th) = (struct slave_host *) malloc(sizeof(struct slave_host));
-	if (!*th) {
-	    fprintf(stderr, "No memory reading host list from '%s'.\n",
+	if (!*th)
+	    errx (1, "No memory reading host list from '%s'.",
 		    file);
-	    exit (1);
-	}
 	memset(*th, 0, sizeof(struct slave_host));
-	(*th)->name = malloc(strlen(namebuf) + 1);
-	if (!(*th)->name) {
-	    fprintf(stderr, "No memory reading host list from '%s'.\n",
-		    file);
-	    exit (1);
-	}
+	(*th)->name = strdup(namebuf);
+	if ((*th)->name == NULL)
+	    errx (1, "No memory reading host list from '%s'.",
+		  file);
 	/* get kerberos cannonical instance name */
-	strcpy((*th)->name, namebuf);
 	inst = krb_get_phost ((*th)->name);
-	(*th)->instance = malloc(strlen(inst) + 1);
-	if (!(*th)->instance) {
-	    fprintf(stderr, "No memory reading host list from '%s'.\n",
-		    file);
-	    exit (1);
-	}
-	strcpy((*th)->instance, inst);
+	(*th)->instance = strdup(inst);
+	if ((*th)->instance == NULL)
+	    errx (1, "No memory reading host list from '%s'.",
+		  file);
 	/* what a concept, slave servers in different realms! */
 	(*th)->realm = my_realm;
 	memcpy(&(*th)->net_addr, host->h_addr, sizeof((*th)->net_addr));
@@ -150,7 +138,7 @@ static int
 prop_to_slaves(struct slave_host *sl, int fd, char *fslv)
 {
     u_char buf[KPROP_BUFSIZ];
-    u_char obuf[KPROP_BUFSIZ + 64	/* leave room for private msg overhead */ ];
+    u_char obuf[KPROP_BUFSIZ + 64]; /* leave room for private msg overhead */
     struct sockaddr_in sin, my_sin;
     int     i, n, s;
     struct slave_host *cs;	/* current slave */
@@ -167,12 +155,12 @@ prop_to_slaves(struct slave_host *sl, int fd, char *fslv)
     
     des_key_schedule session_sched;
 
-    mktemp(tkstring);
+    close(mkstemp(tkstring));
     krb_set_tkt_string(tkstring);
     
     memset(&sin, 0, sizeof sin);
     sin.sin_family = AF_INET;
-    sin.sin_port = k_getportbyname ("krb_prop", "tcp", htons(754));
+    sin.sin_port = k_getportbyname ("krb_prop", "tcp", htons(KPROP_PORT));
     sin.sin_addr.s_addr = INADDR_ANY;
 
     strcpy(path, fslv);
@@ -185,16 +173,13 @@ prop_to_slaves(struct slave_host *sl, int fd, char *fslv)
     for (i = 0; i < 5; i++) {	/* try each slave five times max */
 	for (cs = sl; cs; cs = cs->next) {
 	    if (!cs->succeeded) {
-		if ((s = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
-		    perror("kprop: socket");
-		    exit(1);
-		}
+		if ((s = socket(AF_INET, SOCK_STREAM, 0)) < 0)
+		    err (1, "socket");
 		memcpy(&sin.sin_addr, &cs->net_addr, 
 		      sizeof cs->net_addr);
 
 		if (connect(s, (struct sockaddr *) &sin, sizeof sin) < 0) {
-		    fprintf(stderr, "%s: ", cs->name);
-		    perror("connect");
+		    warn ("connect(%s)", cs->name);
 		    close(s);
 		    continue;	/*** NEXT SLAVE ***/
 		}
@@ -203,13 +188,12 @@ prop_to_slaves(struct slave_host *sl, int fd, char *fslv)
 		memset(&my_sin, 0, sizeof my_sin);
 		n = sizeof my_sin;
 		if (getsockname (s, (struct sockaddr *) &my_sin, &n) != 0) {
-		    fprintf (stderr, "kprop: can't get socketname.");
-		    perror ("getsockname");
+		    warn ("getsockname(%s)", cs->name);
 		    close (s);
 		    continue;	/*** NEXT SLAVE ***/
 		}
 		if (n != sizeof (my_sin)) {
-		    fprintf (stderr, "kprop: can't get socketname. len");
+		    warnx ("can't get socketname %s length", cs->name);
 		    close (s);
 		    continue;	/*** NEXT SLAVE ***/
 		}
@@ -221,9 +205,15 @@ prop_to_slaves(struct slave_host *sl, int fd, char *fslv)
 		 * first get a TGT ...
 		 */
 		if (kerror != MK_AP_OK) {
-		    if (k_gethostname (my_host_name, sizeof(my_host_name)) != 0) {
-			fprintf (stderr, "%s:", cs->name);
-			perror ("getting my hostname");
+		    if (gethostname (my_host_name, sizeof(my_host_name)) != 0) {
+			warnx ("gethostname(%s): %s",
+			       my_host_name,
+#ifdef HAVE_H_ERRNO
+			       hstrerror(h_errno)
+#else
+			       "unknown error"
+#endif			       
+			       );
 			close (s);
 			break;	/* next one can't work either! */
 		    }
@@ -244,9 +234,8 @@ prop_to_slaves(struct slave_host *sl, int fd, char *fslv)
 						 96,
 						 KPROP_SRVTAB);
 		    if (kerror != INTK_OK) {
-			fprintf (stderr,
-				 "%s: %s.  While getting initial ticket\n",
-				 cs->name, krb_get_err_text(kerror));
+			warnx ("%s: %s.  While getting initial ticket\n",
+			       cs->name, krb_get_err_text(kerror));
 			close (s);
 			goto punt;
 		    }
@@ -254,16 +243,15 @@ prop_to_slaves(struct slave_host *sl, int fd, char *fslv)
 					 cs->instance, cs->realm, (u_int32_t) 0);
 		}
 		if (kerror != MK_AP_OK) {
-		    fprintf (stderr, "%s: %s. Calling krb_mk_req.",
-			     cs->name, krb_get_err_text(kerror));
+		    warnx ("%s: krb_mk_req: %s",
+			   cs->name, krb_get_err_text(kerror));
 		    close (s);
 		    continue;	/*** NEXT SLAVE ***/
 		}		    
 
 		if (write(s, kprop_version, sizeof(kprop_version))
 		    != sizeof(kprop_version)) {
-		    fprintf (stderr, "%s: ", cs->name);
-		    perror ("write (version) error");
+		    warn ("%s", cs->name);
 		    close (s);
 		    continue;	/*** NEXT SLAVE ***/
 		}
@@ -271,8 +259,7 @@ prop_to_slaves(struct slave_host *sl, int fd, char *fslv)
 		net_transfer_mode = htons (transfer_mode);
 		if (write(s, &net_transfer_mode, sizeof(net_transfer_mode))
 		    != sizeof(net_transfer_mode)) {
-		    fprintf (stderr, "%s: ", cs->name);
-		    perror ("write (transfer_mode) error");
+		    warn ("write(%s)", cs->name);
 		    close (s);
 		    continue;	/*** NEXT SLAVE ***/
 		}
@@ -280,8 +267,8 @@ prop_to_slaves(struct slave_host *sl, int fd, char *fslv)
 		kerror = krb_get_cred (KPROP_SERVICE_NAME, cs->instance,
 				       cs->realm, &cred);
 		if (kerror != KSUCCESS) {
-		    fprintf (stderr, "%s: %s.  Getting session key.", 
-			     cs->name, krb_get_err_text(kerror));
+		    warnx ("%s: %s.  Getting session key.", 
+			   cs->name, krb_get_err_text(kerror));
 		    close (s);
 		    continue;	/*** NEXT SLAVE ***/
 		}
@@ -289,8 +276,8 @@ prop_to_slaves(struct slave_host *sl, int fd, char *fslv)
 		memset(session_sched, 0, sizeof(session_sched));
 #else
 		if (des_key_sched (&cred.session, session_sched)) {
-		    fprintf (stderr, "%s: can't make key schedule.",
-			     cs->name);
+		    warnx ("%s: can't make key schedule.",
+			   cs->name);
 		    close (s);
 		    continue;	/*** NEXT SLAVE ***/
 		}
@@ -323,18 +310,16 @@ prop_to_slaves(struct slave_host *sl, int fd, char *fslv)
 				      &sin,
 				      KPROP_PROT_VERSION);
 		if (kerror != KSUCCESS) {
-		    fprintf (stderr, "%s: %s.  Calling krb_sendauth.", 
-			     cs->name, krb_get_err_text(kerror));
+		    warnx ("%s: krb_sendauth: %s.",
+			   cs->name, krb_get_err_text(kerror));
 		    close (s);
 		    continue;	/*** NEXT SLAVE ***/
 		}
 
 		lseek(fd, 0L, SEEK_SET); /* Rewind file before rereading it. */
 		while ((n = read(fd, buf, sizeof buf))) {
-		    if (n < 0) {
-			perror("input file read error");
-			exit(1);
-		    }
+		    if (n < 0)
+			err (1, "read");
 		    switch (transfer_mode) {
 		    case KPROP_TRANSFER_PRIVATE:
 		    case KPROP_TRANSFER_SAFE:
@@ -347,32 +332,29 @@ prop_to_slaves(struct slave_host *sl, int fd, char *fslv)
 						  &cred.session,
 						  &my_sin, &sin);
 			if (length == -1) {
-			    fprintf (stderr, "%s: %s failed.",
-				     cs->name,
-				     (transfer_mode == KPROP_TRANSFER_PRIVATE) 
-				     ? "krb_rd_priv" : "krb_rd_safe");
+			    warnx ("%s: %s failed.",
+				   cs->name,
+				   (transfer_mode == KPROP_TRANSFER_PRIVATE) 
+				   ? "krb_rd_priv" : "krb_rd_safe");
 			    close (s);
 			    continue; /*** NEXT SLAVE ***/
 			}
 			nlength = htonl(length);
 			if (write(s, &nlength, sizeof nlength)
 			    != sizeof nlength) {
-			    fprintf (stderr, "%s: ", cs->name);
-			    perror ("write error");
+			    warn ("write(%s)", cs->name);
 			    close (s);
 			    continue; /*** NEXT SLAVE ***/
 			}
 			if (write(s, obuf, length) != length) {
-			    fprintf(stderr, "%s: ", cs->name);
-			    perror("write error");
+			    warn ("write(%s)", cs->name);
 			    close(s);
 			    continue; /*** NEXT SLAVE ***/
 			}
 			break;
 		    case KPROP_TRANSFER_CLEAR:
 			if (write(s, buf, n) != n) {
-			    fprintf(stderr, "%s: ", cs->name);
-			    perror("write error");
+			    warn ("write(%s)", cs->name);
 			    close(s);
 			    continue; /*** NEXT SLAVE ***/
 			}
@@ -425,6 +407,8 @@ main(int argc, char **argv)
     static struct slave_host *slave_host_list = NULL;
     struct slave_host *sh;
 
+    set_progname (argv[0]);
+
     transfer_mode = KPROP_TRANSFER_PRIVATE;
 
     time(&l_init);
@@ -436,7 +420,7 @@ main(int argc, char **argv)
     fslv = NULL;
 
     if (krb_get_lrealm(my_realm,1) != KSUCCESS)
-      Death ("Getting my kerberos realm.  Check krb.conf");
+      errx (1, "Getting my kerberos realm.  Check krb.conf");
 
     for (i = 1; i < argc; i++) 
       switch (argv[i][0]) {
@@ -458,9 +442,8 @@ main(int argc, char **argv)
 	} else if (strcmp (argv[i], "-force") == 0)
 	    force_flag++;
 	else {
-	  fprintf (stderr, "kprop: unknown control argument %s.\n",
-		   argv[i]);
-	  usage ();
+	    warnx("unknown control argument %s.", argv[i]);
+	    usage ();
 	}
 	break;
       default:
@@ -479,34 +462,22 @@ main(int argc, char **argv)
     if(fslv == NULL)
 	fslv = DB_DIR "/slaves";
 	
-    if ((floc_ok = (char *) malloc(strlen(floc) + strlen(ok) + 1))
-	== NULL) {
-	Death(floc);
-    }
-    strcat(strcpy(floc_ok, floc), ok);
+    asprintf (&floc_ok, "%s%s", floc, ok);
+    if (floc_ok == NULL)
+	errx (1, "out of memory in copying %s", floc);
 
-    if ((fd = open(floc, O_RDONLY)) < 0) {
-	Death(floc);
-    }
-    if (k_flock(fd, K_LOCK_EX | K_LOCK_NB)) {
-	Death(floc);
-    }
-    if (stat(floc, &stbuf)) {
-	Death(floc);
-    }
-    if (stat(floc_ok, &stbuf_ok)) {
-	Death(floc_ok);
-    }
-    if (stbuf.st_mtime > stbuf_ok.st_mtime) {
-	fprintf(stderr, "kprop: '%s' more recent than '%s'.\n",
-		floc, floc_ok);
-	exit(1);
-    }
-    if (!get_slaves(&slave_host_list, fslv, stbuf_ok.st_mtime)) {
-	fprintf(stderr,
-		"kprop: can't read slave host file '%s'.\n", fslv);
-	exit(1);
-    }
+    if ((fd = open(floc, O_RDONLY)) < 0)
+	err (1, "open(%s)", floc);
+    if (flock(fd, K_LOCK_SH | K_LOCK_NB))
+	err (1, "flock(%s)", floc);
+    if (stat(floc, &stbuf))
+	err (1, "stat(%s)", floc);
+    if (stat(floc_ok, &stbuf_ok))
+	err (1, "stat(%s)", floc_ok);
+    if (stbuf.st_mtime > stbuf_ok.st_mtime)
+	errx (1, "'%s' more recent than '%s'.", floc, floc_ok);
+    if (!get_slaves(&slave_host_list, fslv, stbuf_ok.st_mtime))
+	errx (1, "can't read slave host file '%s'.", fslv);
 #ifdef KPROP_DBG
     {
 	struct slave_host *sh;
@@ -521,14 +492,10 @@ main(int argc, char **argv)
     }
 #endif				/* KPROP_DBG */
 
-    if (!prop_to_slaves(slave_host_list, fd, fslv)) {
-	fprintf(stderr,
-		"kprop: propagation failed.\n");
-	exit(1);
-    }
-    if (k_flock(fd, K_LOCK_UN)) {
-	Death(floc);
-    }
+    if (!prop_to_slaves(slave_host_list, fd, fslv))
+	errx (1, "propagation failed.");
+    if (flock(fd, K_LOCK_UN))
+	err (1, "flock(%s, LOCK_UN)", floc);
     fprintf(stderr, "\n\n");
     for (sh = slave_host_list; sh; sh = sh->next) {
 	fprintf(stderr, "%s:\t\t%s\n", sh->name,
@@ -554,11 +521,8 @@ u_long get_data_checksum(fd, key_sched)
 	u_int32_t obuf[2];
 
 	while (n = read(fd, buf, sizeof buf)) {
-	    if (n < 0) {
-		fprintf(stderr, "Input data file read error: ");
-		perror("read");
-		exit(1);
-	    }
+	    if (n < 0)
+		err (1, "read");
 	    cksum = cbc_cksum(buf, obuf, n, key_sched, key_sched);
 	}
 	return cksum;
