@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 1995, 1996 Kungliga Tekniska Hgskolan (Royal Institute
- * of Technology, Stockholm, Sweden).
+ * Copyright (c) 1995, 1996, 1997 Kungliga Tekniska Hgskolan
+ * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
  * Redistribution and use in source and binary forms, with or without
@@ -37,12 +37,10 @@
  */
 
 #include "slav_locl.h"
-#include <stdarg.h>
-#include <sys/wait.h>
 
 #include "kprop.h"
 
-RCSID("$Id: kpropd.c,v 2.14 1996/11/15 18:39:32 bg Exp $");
+RCSID("$Id: kpropd.c,v 2.24 1997/12/05 01:34:57 assar Exp $");
 
 #ifndef SBINDIR
 #define SBINDIR "/usr/athena/sbin"
@@ -64,18 +62,6 @@ char *srvtab = "";
 
 char realm[REALM_SZ];
 
-static
-void
-x_log(char *fmt, ...)
-{
-    char buf[1024];
-    va_list ap;
-    va_start(ap, fmt);
-    vsprintf(buf, fmt, ap);
-    klog(L_KRB_PERR, buf);
-    va_end(ap);
-}
-
 static
 int
 copy_data(int from, int to, des_cblock *session, des_key_schedule schedule)
@@ -93,25 +79,25 @@ copy_data(int from, int to, des_cblock *session, des_key_schedule schedule)
 	if(n == 0)
 	    break;
 	if(n < 0){
-	    x_log("krb_net_read: %s", strerror(errno));
+	    klog(L_KRB_PERR, "krb_net_read: %s", strerror(errno));
 	    return -1;
 	}
 	if(n != 4){
-	    x_log("Premature end of data");
+	    klog(L_KRB_PERR, "Premature end of data");
 	    return -1;
 	}
 	length = (tmp[0] << 24) | (tmp[1] << 16) | (tmp[2] << 8) | tmp[3];
 	if(length > sizeof(buf)){
-	    x_log("Giant packet received: %d", length);
+	    klog(L_KRB_PERR, "Giant packet received: %d", length);
 	    return -1;
 	}
 	if(krb_net_read(from, buf, length) != length){
-	    x_log("Premature end of data");
+	    klog(L_KRB_PERR, "Premature end of data");
 	    return -1;
 	}
 	kerr = krb_rd_priv (buf, length, schedule, session, &master, &slave, &m);
 	if(kerr != KSUCCESS){
-	    x_log("Kerberos error: %s", krb_get_err_text(kerr));
+	    klog(L_KRB_PERR, "Kerberos error: %s", krb_get_err_text(kerr));
 	    return -1;
 	}
 	write(to, m.app_data, m.app_length);
@@ -137,58 +123,71 @@ kprop(int s)
     
     n = sizeof(master);
     if(getpeername(s, (struct sockaddr*)&master, &n) < 0){
-	x_log("getpeername: %s", strerror(errno));
+	klog(L_KRB_PERR, "getpeername: %s", strerror(errno));
 	return 1;
     }
     
     n = sizeof(slave);
     if(getsockname(s, (struct sockaddr*)&slave, &n) < 0){
-	x_log("getsockname: %s", strerror(errno));
+	klog(L_KRB_PERR, "getsockname: %s", strerror(errno));
 	return 1;
     }
 
-    x_log("Connection from %s", inet_ntoa(master.sin_addr));
+    klog(L_KRB_PERR, "Connection from %s", inet_ntoa(master.sin_addr));
 
     n = krb_net_read(s, buf, KPROP_PROT_VERSION_LEN + 2);
     if(n < KPROP_PROT_VERSION_LEN + 2){
-	x_log("Premature end of data");
+	klog(L_KRB_PERR, "Premature end of data");
 	return 1;
     }
     if(memcmp(buf, KPROP_PROT_VERSION, KPROP_PROT_VERSION_LEN) != 0){
-	x_log("Bad protocol version string received");
+	klog(L_KRB_PERR, "Bad protocol version string received");
 	return 1;
     }
     mode = (buf[n-2] << 8) | buf[n-1];
     if(mode != KPROP_TRANSFER_PRIVATE){
-	x_log("Bad transfer mode received: %d", mode);
+	klog(L_KRB_PERR, "Bad transfer mode received: %d", mode);
 	return 1;
     }
-    k_getsockinst(s, sinst);
+    k_getsockinst(s, sinst, sizeof(sinst));
     kerr = krb_recvauth(KOPT_DO_MUTUAL, s, &ticket,
 			KPROP_SERVICE_NAME, sinst,
 			&master, &slave,
 			&ad, srvtab, schedule, 
 			buf);
     if(kerr != KSUCCESS){
-	x_log("Kerberos error: %s", krb_get_err_text(kerr));
+	klog(L_KRB_PERR, "Kerberos error: %s", krb_get_err_text(kerr));
 	return 1;
     }
+
+    if(strcmp(ad.pname, KPROP_SERVICE_NAME) ||
+#if 0
+       strcmp(ad.pinst, /* XXX remote host */) ||
+#else
+       strcmp(ad.pinst, KRB_MASTER) ||
+#endif
+       strcmp(ad.prealm, realm)){
+	klog(L_KRB_PERR, "Connection from unauthorized client: %s", 
+	     krb_unparse_name_long(ad.pname, ad.pinst, ad.prealm));
+	return 1;
+    }
+
     des_set_key(&ad.session, schedule);
     
     lock = open(lockfile, O_WRONLY|O_CREAT, 0600);
     if(lock < 0){
-	x_log("Failed to open file: %s", strerror(errno));
+	klog(L_KRB_PERR, "Failed to open file: %s", strerror(errno));
 	return 1;
     }
-    if(k_flock(lock, K_LOCK_EX | K_LOCK_NB)){
+    if(flock(lock, K_LOCK_EX | K_LOCK_NB)){
 	close(lock);
-	x_log("Failed to lock file: %s", strerror(errno));
+	klog(L_KRB_PERR, "Failed to lock file: %s", strerror(errno));
 	return 1;
     }
     
     if(ftruncate(lock, 0) < 0){
 	close(lock);
-	x_log("Failed to lock file: %s", strerror(errno));
+	klog(L_KRB_PERR, "Failed to lock file: %s", strerror(errno));
 	return 1;
     }
 
@@ -197,84 +196,87 @@ kprop(int s)
 	return 1;
     }
     close(lock);
-    sprintf(command, "%s %s %s %s", kdb_util, kdb_util_command, 
+    snprintf(command, sizeof(command),
+	     "%s %s %s %s", kdb_util, kdb_util_command, 
 	    lockfile, database);
     if(system(command) == 0){
-	x_log("Propagation finished successfully");
+	klog(L_KRB_PERR, "Propagation finished successfully");
 	return 0;
     }
-    x_log("*** Propagation failed ***");
+    klog(L_KRB_PERR, "*** Propagation failed ***");
     return 1;
 }
 
-static
-int
+static int
 doit(void)
+{
+    return kprop(0);
+}
+
+static int
+doit_interactive(void)
 {
     struct sockaddr_in sa;
     int salen;
     int s, s2;
     int ret;
 
-    salen = sizeof(sa);
-    ret = getpeername(0, (struct sockaddr*)&sa, &salen);
-    if(ret < 0 && (errno == ENOTSOCK || errno == EINVAL)){
-	s = socket(AF_INET, SOCK_STREAM, 0);
-	if(s < 0){
-	    x_log("socket: %s", strerror(errno));
-	    return 1;
-	}
-	memset(&sa, 0, sizeof(sa));
-	sa.sin_family = AF_INET;
-	sa.sin_port = k_getportbyname ("krb_prop", "tcp", htons(754));
-	salen = sizeof(sa);
-	ret = bind(s, (struct sockaddr*)&sa, sizeof(sa));
-	if(ret < 0){
-	    krb_log("bind: %s", strerror(errno));
-	    return 1;
-	}
-	listen(s, 5);
-	while(1){
-	    salen = sizeof(sa);
-	    s2 = accept(s, (struct sockaddr*)&sa, &salen);
-	    switch(fork()){
-	    case -1:
-		x_log("fork: %s", strerror(errno));
-		return 1;
-	    case 0:
-		kprop(s2);
-		return 1;
-	    default: {
-		    int status;
-		    close(s2);
-		    wait(&status);
-		}
-	    }
-	}
+    s = socket(AF_INET, SOCK_STREAM, 0);
+    if(s < 0){
+      klog(L_KRB_PERR, "socket: %s", strerror(errno));
+      return 1;
     }
-    if(ret < 0){
-	x_log("getpeername: %s", strerror(errno));
+    memset(&sa, 0, sizeof(sa));
+    sa.sin_family = AF_INET;
+    sa.sin_port = k_getportbyname ("krb_prop", "tcp", htons(KPROP_PORT));
+    ret = bind(s, (struct sockaddr*)&sa, sizeof(sa));
+    if (ret < 0) {
+      klog(L_KRB_PERR, "bind: %s", strerror(errno));
+      return 1;
+    }
+    ret = listen(s, SOMAXCONN);
+    if (ret < 0) {
+      klog(L_KRB_PERR, "listen: %s", strerror(errno));
+      return 1;
+    }
+    for(;;) {
+      salen = sizeof(sa);
+      s2 = accept(s, (struct sockaddr*)&sa, &salen);
+      switch(fork()){
+      case -1:
+	klog(L_KRB_PERR, "fork: %s", strerror(errno));
 	return 1;
+      case 0:
+	close(s);
+	kprop(s2);
+	return 1;
+      default: {
+	  int status;
+	  close(s2);
+	  wait(&status);
+	}
+      }
     }
-    return kprop(0);
 }
 
 static void
 usage (void)
 {
      fprintf (stderr, 
-	      "Usage: kpropd [-d database] [-l log] [-m] [-[p|P] program] "
-	      "[-r realm] [-s srvtab]\n");
+	      "Usage: kpropd [-i] [-d database] [-l log] [-m] [-[p|P] program]"
+	      " [-r realm] [-s srvtab]\n");
      exit (1);
 }
 
-int main(int argc, char **argv)
+int
+main(int argc, char **argv)
 {
     int opt;
+    int interactive = 0;
 
     krb_get_lrealm(realm, 1);
     
-    while((opt = getopt(argc, argv, ":d:l:mp:P:r:s:")) >= 0){
+    while((opt = getopt(argc, argv, ":d:l:mp:P:r:s:i")) >= 0){
 	switch(opt){
 	case 'd':
 	    database = optarg;
@@ -295,12 +297,18 @@ int main(int argc, char **argv)
 	case 's':
 	    srvtab = optarg;
 	    break;
+	case 'i':
+	    interactive = 1;
+	    break;
 	default:
-	    x_log("Bad option: -%c", optopt);
+	    klog(L_KRB_PERR, "Bad option: -%c", optopt);
 	    usage ();
 	    exit(1);
 	}
     }
     kset_logfile(logfile);
-    return doit();
+    if (interactive)
+      return doit_interactive ();
+    else
+      return doit ();
 }
